"""System module."""
from __future__ import absolute_import

class Proxy():
    '''Proxy class'''
    @staticmethod
    def set_proxy(url):
        '''Set the proxy'''
        http_proxy  = "<proxy>"
        proxy_dict = {
              "http"  : http_proxy,
              "https" : http_proxy
        }
        return_value=None
        if url == "<proxy>":
            return_value = proxy_dict
        elif url == "<proxy>":
            return_value = proxy_dict
        return return_value



"""System module."""
from __future__ import absolute_import
from selenium.webdriver.common.keys import Keys

class EventKey() :
    ''' Event Key '''
    @staticmethod
    def get_key (key):
        ''' get_key'''
        keys_array= {
            "KEY_ADD": Keys.ADD,
            "KEY_ALT": Keys.ALT,
            "KEY_ARROW_DOWN": Keys.ARROW_DOWN,
            "KEY_ARROW_LEFT": Keys.ARROW_LEFT,
            "KEY_ARROW_RIGHT": Keys.ARROW_RIGHT,
            "KEY_ARROW_UP": Keys.ARROW_UP,
            "KEY_BACKSPACE": Keys.BACKSPACE,
            "KEY_BACK_SPACE": Keys.BACK_SPACE,
            "KEY_CANCEL": Keys.CANCEL,
            "KEY_CLEAR": Keys.CLEAR,
            "KEY_COMMAND": Keys.COMMAND,
            "KEY_CONTROL": Keys.CONTROL,
            "KEY_DECIMAL": Keys.DECIMAL,
            "KEY_DELETE": Keys.DELETE,
            "KEY_DIVIDE": Keys.DIVIDE,
            "KEY_DOWN": Keys.DOWN,
            "KEY_END": Keys.END,
            "KEY_ENTER": Keys.ENTER,
            "KEY_EQUALS": Keys.EQUALS,
            "KEY_ESCAPE": Keys.ESCAPE,
            "KEY_F1": Keys.F1,
            "KEY_F10": Keys.F10,
            "KEY_F11": Keys.F11,
            "KEY_F12": Keys.F12,
            "KEY_F2": Keys.F2,
            "KEY_F3": Keys.F3,
            "KEY_F4": Keys.F4,
            "KEY_F5": Keys.F5,
            "KEY_F6": Keys.F6,
            "KEY_F7": Keys.F7,
            "KEY_F8": Keys.F8,
            "KEY_F9": Keys.F9,
            "KEY_HELP": Keys.HELP,
            "KEY_HOME": Keys.HOME,
            "KEY_INSERT": Keys.INSERT,
            "KEY_LEFT": Keys.LEFT,
            "KEY_LEFT_ALT": Keys.LEFT_ALT,
            "KEY_LEFT_CONTROL": Keys.LEFT_CONTROL,
            "KEY_LEFT_SHIFT": Keys.LEFT_SHIFT,
            "KEY_META": Keys.META,
            "KEY_MULTIPLY": Keys.MULTIPLY,
            "KEY_NULL": Keys.NULL,
            "KEY_NUMPAD0": Keys.NUMPAD0,
            "KEY_NUMPAD1": Keys.NUMPAD1,
            "KEY_NUMPAD2": Keys.NUMPAD2,
            "KEY_NUMPAD3": Keys.NUMPAD3,
            "KEY_NUMPAD4": Keys.NUMPAD4,
            "KEY_NUMPAD5": Keys.NUMPAD5,
            "KEY_NUMPAD6": Keys.NUMPAD6,
            "KEY_NUMPAD7": Keys.NUMPAD7,
            "KEY_NUMPAD8": Keys.NUMPAD8,
            "KEY_NUMPAD9": Keys.NUMPAD9,
            "KEY_PAGE_DOWN": Keys.PAGE_DOWN,
            "KEY_PAGE_UP": Keys.PAGE_UP,
            "KEY_PAUSE": Keys.PAUSE,
            "KEY_RETURN": Keys.RETURN,
            "KEY_RIGHT": Keys.RIGHT,
            "KEY_SEMICOLON": Keys.SEMICOLON,
            "KEY_SEPARATOR": Keys.SEPARATOR,
            "KEY_SHIFT": Keys.SHIFT,
            "KEY_SPACE": Keys.SPACE,
            "KEY_SUBTRACT": Keys.SUBTRACT,
            "KEY_TAB": Keys.TAB,
            "KEY_UP": Keys.UP,
        }
        if key in keys_array.keys():
            return keys_array[key]
        return None



"""System module."""
from __future__ import absolute_import
import sys
import ssl
import os
import warnings
from typing import List, Optional

from http.server import HTTPServer

from idelium._internal.ideliummanager import StartManager
from idelium._internal.ideliumserver import IdeliumServer
from idelium._internal.ideliumws import IdeliumWs
from idelium._internal.ideliumclib import InitIdelium
from idelium._internal.thirdparties.ideliumzephyr import ZephyrConnection
from idelium._internal.commons.ideliumprinter import InitPrinter


idelium=StartManager()
printer=InitPrinter()
ideliumws=IdeliumWs()
idelium_cl_lib=InitIdelium()
IDELIUM_VERSION='1.0.5'

def main(args: Optional[List[str]] = None) -> int:
    printer.print_important_text("Idelium Command Line " + IDELIUM_VERSION)
    printer.print_important_text ("Selenium version:" + idelium_cl_lib.get_selenium_version())
    if args is None:
        args = sys.argv
    define_parameters= idelium_cl_lib.define_parameters(args,ideliumws,printer)
    cl_params=define_parameters['cl_params']

    if cl_params['ideliumServer'] is False:
        define_parameters = idelium_cl_lib.load_parameters(cl_params, ideliumws, printer)
        cl_params = define_parameters['cl_params']
        test_config = define_parameters['test_config']
        if cl_params['reportingService'] == 'idelium':
            ideliumws.start_test(idelium,test_config,cl_params)
        elif cl_params['reportingService'] == 'zephyr':
            zephyr=ZephyrConnection()
            if cl_params['idJira'] is not None:
                zephyr.start_test_case(idelium,test_config,cl_params)
            else:
                zephyr.go_execution(idelium,cl_params)
        else:
            printer.danger('Error: ' + cl_params['reportingService'] + ' has a wrong value')
        printer.success('Finish test')
    else:
        if os.path.exists(cl_params['dir_idelium_scripts'] + 'server'):
            os.remove(cl_params['dir_idelium_scripts'] + 'server')
        server_address = ('0.0.0.0', cl_params['ideliumServerPort'])
        IdeliumServer.init(idelium, cl_params, ideliumws, idelium_cl_lib,printer)
        sslctx = ssl.SSLContext()
        sslctx.check_hostname = False
        sslctx.load_cert_chain(certfile='cert/cert.pem', keyfile="cert/key.pem")
        httpd = HTTPServer(server_address, IdeliumServer)
        httpd.socket = sslctx.wrap_socket(httpd.socket, server_side=True)
        printer.success('Server start on port:' +
            str(cl_params['ideliumServerPort']))
        httpd.serve_forever()



import difflib
import json
import re
from copy import copy

import requests
from mergedeep import merge

from postpy2.extractors import extract_dict_from_raw_headers, extract_dict_from_headers, extract_dict_from_raw_mode_data, format_object, extract_dict_from_formdata_mode_data, exctact_dict_from_files


class CaseSensitiveDict(dict):

    def update(self, d=None, **kwargs):
        d = d or {}
        for k, v in d.items():
            self[k] = v

    def loadOld(self, postman_enviroment_file_path):
        with open(postman_enviroment_file_path, encoding='utf8') as postman_enviroment_file:
            postman_enviroment = json.load(postman_enviroment_file)
            for item in postman_enviroment['values']:
                if item['enabled']:
                    self[item['key']] = item['value']

    def load(self, postman_enviroment):
        for item in postman_enviroment['values']:
            if item['enabled']:
                    self[item['key']] = item['value']


class PostPython:
    ''' 
        modify for idelium 
        def __init__(self, postman_collection_file_path, request_overrides=None):
        with open(postman_collection_file_path, encoding='utf8') as postman_collection_file:
            self.__postman_collection = json.load(postman_collection_file)'''
    def __init__(self, collection, request_overrides=None):
        self.__postman_collection = collection
        self.__folders = {}
        self.environments = CaseSensitiveDict()
        self.request_overrides = request_overrides
        self.__load()

    def __load(self):
        for fol in self.__postman_collection['item']:
            requests_list = {}
            print (fol['name'])
            for request in fol['item']:
                if 'request' in request:
                    request['request']['name'] = request['name']
                    requests_list[normalize_func_name(
                        request['name'])] = PostRequest(self, request['request'])

            col_name = normalize_class_name(fol['name'])
            self.__folders[col_name] = PostCollection(col_name, requests_list)

    def __getattr__(self, item):
        if item in self.__folders:
            return self.__folders[item]
        else:
            folders = list(self.__folders.keys())
            similar_folders = difflib.get_close_matches(item, folders)
            if len(similar_folders) > 0:
                similar = similar_folders[0]
                raise AttributeError('%s folder does not exist in Postman collection.\n'
                                     'Did you mean %s?' % (item, similar))
            else:
                raise AttributeError('%s folder does not exist in Postman collection.\n'
                                     'Your choices are: %s' % (item, ", ".join(folders)))

    def help(self):
        print("Possible methods:")
        for fol in self.__folders.values():
            print()
            fol.help()
            
    def getmethods(self):
        methods=[]
        for fol in self.__folders.values():
            methods.append(fol.gethelp())
        return methods

class PostCollection:
    def __init__(self, name, requests_list):
        self.name = name
        self.__requests = requests_list

    def __getattr__(self, item):
        if item in self.__requests:
            return self.__requests[item]
        else:
            post_requests = list(self.__requests.keys())
            similar_requests = difflib.get_close_matches(
                item, post_requests, cutoff=0.0)
            if len(similar_requests) > 0:
                similar = similar_requests[0]
                raise AttributeError('%s request does not exist in %s folder.\n'
                                     'Did you mean %s' % (item, self.name, similar))
            else:
                raise AttributeError('%s request does not exist in %s folder.\n'
                                     'Your choices are: %s' % (item, self.name, ", ".join(post_requests)))

    def help(self):
        object_help=[]
        for req in self.__requests.keys():
            print("post_python.{COLLECTION}.{REQUEST}()".format(
                COLLECTION=self.name, REQUEST=req))
    def gethelp(self):
        object_help=[]
        for req in self.__requests.keys():
            object_help.append("post_python.{COLLECTION}.{REQUEST}".format(
                COLLECTION=self.name, REQUEST=req))
        return object_help   


class PostRequest:
    def __init__(self, post_python, data):
        self.name = normalize_func_name(data['name'])
        self.post_python = post_python
        self.request_kwargs = dict()
        self.request_kwargs['url'] = data['url']['raw']
        self.is_graphql = False

        if 'body' in data and data['body']['mode'] == 'raw' and 'raw' in data['body']:
            self.request_kwargs['json'] = extract_dict_from_raw_mode_data(
                data['body']['raw'])

        if 'body' in data and data['body']['mode'] == 'formdata' and 'formdata' in data['body']:
            self.request_kwargs['data'], self.request_kwargs['files'] = extract_dict_from_formdata_mode_data(
                data['body']['formdata'])

        if 'body' in data and data['body']['mode'] == 'graphql':
            self.request_kwargs['json'] = data['body']['graphql']
            if data['body']['graphql']['variables'] == "":
                data['body']['graphql']['variables'] = "{}"
            self.is_graphql = True

        self.request_kwargs['headers'] = extract_dict_from_headers(data['header'])
        self.request_kwargs['method'] = data['method']

    def __call__(self, *args, **kwargs):

        current_request_kwargs = copy(self.request_kwargs)

        if self.post_python.request_overrides:
            current_request_kwargs = merge(current_request_kwargs, self.post_python.request_overrides)
        new_env = copy(self.post_python.environments)
        new_env.update(kwargs)

        if 'files' in current_request_kwargs:
            for key, file in current_request_kwargs['files'].items():
                file[1].seek(0) # flip byte stream for subsequent reads

        formatted_kwargs = format_object(current_request_kwargs, new_env, self.is_graphql)
        print (formatted_kwargs)
        return requests.request(**formatted_kwargs)

    def set_files(self, data):
        for row in data:
            self.request_kwargs['files'][row['key']
                                         ] = exctact_dict_from_files(row)

    def set_data(self, data):
        for row in data:
            self.request_kwargs['data'][row['key']] = row['value']

    def set_json(self, data):
        self.request_kwargs['json'] = data


def normalize_class_name(string):
    string = re.sub(r'[?!@#$%^&*()_\-+=,./\'\\\"|:;{}\[\]]', ' ', string)
    return string.title().replace(' ', '')


def normalize_func_name(string):
    string = re.sub(r'[?!@#$%^&*()_\-+=,./\'\\\"|:;{}\[\]]', ' ', string)
    return '_'.join(string.lower().split())



"""System module."""
from __future__ import absolute_import
class EventKey():
    '''Event Key'''
    @staticmethod
    def key_event_table (char):
        ''' key_event_table '''
        char_list= {
                "KEYEVENT_UNKNOWN": "0",
                "KEYEVENT_SOFT_LEFT": "1",
                "KEYEVENT_SOFT_RIGHT": "2",
                "KEYEVENT_HOME": "3",
                "KEYEVENT_BACK": "4",
                "KEYEVENT_CALL": "5",
                "KEYEVENT_ENDCALL": "6",
                "KEYEVENT_0": "7",
                "KEYEVENT_1": "8",
                "KEYEVENT_2": "9",
                "KEYEVENT_3": "10",
                "KEYEVENT_4": "11",
                "KEYEVENT_5": "12",
                "KEYEVENT_6": "13",
                "KEYEVENT_7": "14",
                "KEYEVENT_8": "15",
                "KEYEVENT_9": "16",
                "KEYEVENT_STAR": "17",
                "KEYEVENT_POUND": "18",
                "KEYEVENT_DPAD_UP": "19",
                "KEYEVENT_DPAD_DOWN": "20",
                "KEYEVENT_DPAD_LEFT": "21",
                "KEYEVENT_DPAD_RIGHT": "22",
                "KEYEVENT_DPAD_CENTER": "23",
                "KEYEVENT_VOLUME_UP": "24",
                "KEYEVENT_VOLUME_DOWN": "25",
                "KEYEVENT_POWER": "26",
                "KEYEVENT_CAMERA": "27",
                "KEYEVENT_CLEAR": "28",
                "KEYEVENT_A": "29",
                "KEYEVENT_B": "30",
                "KEYEVENT_C": "31",
                "KEYEVENT_D": "32",
                "KEYEVENT_E": "33",
                "KEYEVENT_F": "34",
                "KEYEVENT_G": "35",
                "KEYEVENT_H": "36",
                "KEYEVENT_I": "37",
                "KEYEVENT_J": "38",
                "KEYEVENT_K": "39",
                "KEYEVENT_L": "40",
                "KEYEVENT_M": "41",
                "KEYEVENT_N": "42",
                "KEYEVENT_O": "43",
                "KEYEVENT_P": "44",
                "KEYEVENT_Q": "45",
                "KEYEVENT_R": "46",
                "KEYEVENT_S": "47",
                "KEYEVENT_T": "48",
                "KEYEVENT_U": "49",
                "KEYEVENT_V": "50",
                "KEYEVENT_W": "51",
                "KEYEVENT_X": "52",
                "KEYEVENT_Y": "53",
                "KEYEVENT_Z": "54",
                "KEYEVENT_,": "55",
                "KEYEVENT_PERIOD": "56",
                "KEYEVENT_ALT_LEFT": "57",
                "KEYEVENT_ALT_RIGHT": "58",
                "KEYEVENT_SHIFT_LEFT": "59",
                "KEYEVENT_SHIFT_RIGHT": "60",
                "KEYEVENT_TAB": "61",
                "KEYEVENT_SPACE": "62",
                "KEYEVENT_SYM": "63",
                "KEYEVENT_EXPLORER": "64",
                "KEYEVENT_ENVELOPE": "65",
                "KEYEVENT_ENTER": "66",
                "KEYEVENT_DEL": "67",
                "KEYEVENT_GRAVE": "68",
                "KEYEVENT_MINUS": "69",
                "KEYEVENT_EQUALS": "70",
                "KEYEVENT_LEFT_BRACKET": "71",
                "KEYEVENT_RIGHT_BRACKET": "72",
                "KEYEVENT_BACKSLASH": "73",
                "KEYEVENT_SEMICOLON": "74",
                "KEYEVENT_APOSTROPHE": "75",
                "KEYEVENT_SLASH": "76",
                "KEYEVENT_AT": "77",
                "KEYEVENT_NUM": "78",
                "KEYEVENT_HEADSETHOOK": "79",
                "KEYEVENT_PLUS": "81",
                "KEYEVENT_MENU": "82",
                "KEYEVENT_NOTIFICATION": "83",
                "KEYEVENT_SEARCH": "84",
                "KEYEVENT_MEDIA_PLAY_PAUSE": "85",
                "KEYEVENT_MEDIA_STOP": "86",
                "KEYEVENT_MEDIA_NEXT": "87",
                "KEYEVENT_MEDIA_PREVIOUS": "88",
                "KEYEVENT_MEDIA_REWIND": "89",
                "KEYEVENT_MEDIA_FAST_FORWARD": "90",
                "KEYEVENT_MUTE": "91",
                "KEYEVENT_PAGE_UP": "92",
                "KEYEVENT_PAGE_DOWN": "93",
                "KEYEVENT_BUTTON_A": "96",
                "KEYEVENT_BUTTON_B": "97",
                "KEYEVENT_BUTTON_C": "98",
                "KEYEVENT_BUTTON_X": "99",
                "KEYEVENT_BUTTON_Y": "100",
                "KEYEVENT_BUTTON_Z": "101",
                "KEYEVENT_BUTTON_L1": "102",
                "KEYEVENT_BUTTON_R1": "103",
                "KEYEVENT_BUTTON_L2": "104",
                "KEYEVENT_BUTTON_R2": "105",
                "KEYEVENT_BUTTON_THUMBL": "106",
                "KEYEVENT_BUTTON_THUMBR": "107",
                "KEYEVENT_BUTTON_START": "108",
                "KEYEVENT_BUTTON_SELECT": "109",
                "KEYEVENT_BUTTON_MODE": "110",
                "KEYEVENT_ESCAPE": "111",
                "KEYEVENT_FORWARD_DEL": "112",
                "KEYEVENT_CTRL_LEFT": "113",
                "KEYEVENT_CTRL_RIGHT": "114",
                "KEYEVENT_CAPS_LOCK": "115",
                "KEYEVENT_SCROLL_LOCK": "116",
                "KEYEVENT_META_LEFT": "117",
                "KEYEVENT_META_RIGHT": "118",
                "KEYEVENT_FUNCTION": "119",
                "KEYEVENT_SYSRQ": "120",
                "KEYEVENT_BREAK": "121",
                "KEYEVENT_MOVE_HOME": "122",
                "KEYEVENT_MOVE_END": "123",
                "KEYEVENT_INSERT": "124",
                "KEYEVENT_FORWARD": "125",
                "KEYEVENT_MEDIA_PLAY": "126",
                "KEYEVENT_MEDIA_PAUSE": "127",
                "KEYEVENT_MEDIA_CLOSE": "128",
                "KEYEVENT_MEDIA_EJECT": "129",
                "KEYEVENT_MEDIA_RECORD": "130",
                "KEYEVENT_F1": "131",
                "KEYEVENT_F2": "132",
                "KEYEVENT_F3": "133",
                "KEYEVENT_F4": "134",
                "KEYEVENT_F5": "135",
                "KEYEVENT_F6": "136",
                "KEYEVENT_F7": "137",
                "KEYEVENT_F8": "138",
                "KEYEVENT_F9": "139",
                "KEYEVENT_F10": "140",
                "KEYEVENT_F11": "141",
                "KEYEVENT_F12": "142",
                "KEYEVENT_NUM_LOCK": "143",
                "KEYEVENT_NUMPAD_0": "144",
                "KEYEVENT_NUMPAD_1": "145",
                "KEYEVENT_NUMPAD_2": "146",
                "KEYEVENT_NUMPAD_3": "147",
                "KEYEVENT_NUMPAD_4": "148",
                "KEYEVENT_NUMPAD_5": "149",
                "KEYEVENT_NUMPAD_6": "150",
                "KEYEVENT_NUMPAD_7": "151",
                "KEYEVENT_NUMPAD_8": "152",
                "KEYEVENT_NUMPAD_9": "153",
                "KEYEVENT_NUMPAD_DIVIDE": "154",
                "KEYEVENT_NUMPAD_MULTIPLY": "155",
                "KEYEVENT_NUMPAD_SUBTRACT": "156",
                "KEYEVENT_NUMPAD_ADD": "157",
                "KEYEVENT_NUMPAD_DOT": "158",
                "KEYEVENT_NUMPAD_COMMA": "159",
                "KEYEVENT_NUMPAD_ENTER": "160",
                "KEYEVENT_NUMPAD_EQUALS": "161",
                "KEYEVENT_NUMPAD_LEFT_PAREN": "162",
                "KEYEVENT_NUMPAD_RIGHT_PAREN": "163",
                "KEYEVENT_VOLUME_MUTE": "164",
                "KEYEVENT_INFO": "165",
                "KEYEVENT_CHANNEL_UP": "166",
                "KEYEVENT_CHANNEL_DOWN": "167",
                "KEYEVENT_ZOOM_IN": "168",
                "KEYEVENT_ZOOM_OUT": "169",
                "KEYEVENT_TV": "170",
                "KEYEVENT_WINDOW": "171",
                "KEYEVENT_GUIDE": "172",
                "KEYEVENT_DVR": "173",
                "KEYEVENT_BOOKMARK": "174",
                "KEYEVENT_CAPTIONS": "175",
                "KEYEVENT_SETTINGS": "176",
                "KEYEVENT_TV_POWER": "177",
                "KEYEVENT_TV_INPUT": "178",
                "KEYEVENT_STB_POWER": "179",
                "KEYEVENT_STB_INPUT": "180",
                "KEYEVENT_AVR_POWER": "181",
                "KEYEVENT_AVR_INPUT": "182",
                "KEYEVENT_PROG_RED": "183",
                "KEYEVENT_PROG_GREEN": "184",
                "KEYEVENT_PROG_YELLOW": "185",
                "KEYEVENT_PROG_BLUE": "186",
                "KEYEVENT_APP_SWITCH": "187",
                "KEYEVENT_BUTTON_1": "188",
                "KEYEVENT_BUTTON_2": "189",
                "KEYEVENT_BUTTON_3": "190",
                "KEYEVENT_BUTTON_4": "191",
                "KEYEVENT_BUTTON_5": "192",
                "KEYEVENT_BUTTON_6": "193",
                "KEYEVENT_META_SHIFT_ON" : "193",
                "KEYEVENT_BUTTON_7": "194",
                "KEYEVENT_BUTTON_8": "195",
                "KEYEVENT_BUTTON_9": "196",
                "KEYEVENT_BUTTON_10": "197",
                "KEYEVENT_BUTTON_11": "198",
                "KEYEVENT_BUTTON_12": "199",
                "KEYEVENT_BUTTON_13": "200",
                "KEYEVENT_BUTTON_14": "201",
                "KEYEVENT_BUTTON_15": "202",
                "KEYEVENT_BUTTON_16": "203",
                "KEYEVENT_LANGUAGE_SWITCH": "204",
                "KEYEVENT_MANNER_MODE": "205",
                "KEYEVENT_3D_MODE": "206",
                "KEYEVENT_CONTACTS": "207",
                "KEYEVENT_CALENDAR": "208",
                "KEYEVENT_MUSIC": "209",
                "KEYEVENT_CALCULATOR": "210",
                "KEYEVENT_ZENKAKU_HANKAKU": "211",
                "KEYEVENT_EISU": "212",
                "KEYEVENT_MUHENKAN": "213",
                "KEYEVENT_HENKAN": "214",
                "KEYEVENT_KATAKANA_HIRAGANA": "215",
                "KEYEVENT_YEN": "216",
                "KEYEVENT_RO": "217",
                "KEYEVENT_KANA": "218",
                "KEYEVENT_ASSIST": "219",
                "KEYEVENT_BRIGHTNESS_DOWN": "220",
                "KEYEVENT_BRIGHTNESS_UP": "221",
                "KEYEVENT_MEDIA_AUDIO_TRACK": "222",
                "KEYEVENT_SLEEP": "223",
                "KEYEVENT_WAKEUP": "224",
                "KEYEVENT_PAIRING": "225",
                "KEYEVENT_MEDIA_TOP_MENU": "226",
                "KEYEVENT_11": "227",
                "KEYEVENT_12": "228",
                "KEYEVENT_LAST_CHANNEL": "229",
                "KEYEVENT_TV_DATA_SERVICE": "230",
                "KEYEVENT_VOICE_ASSIST": "231",
                "KEYEVENT_TV_RADIO_SERVICE": "232",
                "KEYEVENT_TV_TELETEXT": "233",
                "KEYEVENT_TV_NUMBER_ENTRY": "234",
                "KEYEVENT_TV_TERRESTRIAL_ANALOG": "235",
                "KEYEVENT_TV_TERRESTRIAL_DIGITAL": "236",
                "KEYEVENT_TV_SATELLITE": "237",
                "KEYEVENT_TV_SATELLITE_BS": "238",
                "KEYEVENT_TV_SATELLITE_CS": "239",
                "KEYEVENT_TV_SATELLITE_SERVICE": "240",
                "KEYEVENT_TV_NETWORK": "241",
                "KEYEVENT_TV_ANTENNA_CABLE": "242",
                "KEYEVENT_TV_INPUT_HDMI_1": "243",
                "KEYEVENT_TV_INPUT_HDMI_2": "244",
                "KEYEVENT_TV_INPUT_HDMI_3": "245",
                "KEYEVENT_TV_INPUT_HDMI_4": "246",
                "KEYEVENT_TV_INPUT_COMPOSITE_1": "247",
                "KEYEVENT_TV_INPUT_COMPOSITE_2": "248",
                "KEYEVENT_TV_INPUT_COMPONENT_1": "249",
                "KEYEVENT_TV_INPUT_COMPONENT_2": "250",
                "KEYEVENT_TV_INPUT_VGA_1": "251",
                "KEYEVENT_TV_AUDIO_DESCRIPTION": "252",
                "KEYEVENT_TV_AUDIO_DESCRIPTION_MIX_UP": "253",
                "KEYEVENT_TV_AUDIO_DESCRIPTION_MIX_DOWN": "254",
                "KEYEVENT_TV_ZOOM_MODE": "255",
                "KEYEVENT_TV_CONTENTS_MENU": "256",
                "KEYEVENT_TV_MEDIA_CONTEXT_MENU": "257",
                "KEYEVENT_TV_TIMER_PROGRAMMING": "258",
                "KEYEVENT_HELP": "259",
                "KEYEVENT_NAVIGATE_PREVIOUS": "260",
                "KEYEVENT_NAVIGATE_NEXT": "261",
                "KEYEVENT_NAVIGATE_IN": "262",
                "KEYEVENT_NAVIGATE_OUT": "263",
                "KEYEVENT_STEM_PRIMARY": "264",
                "KEYEVENT_STEM_1": "265",
                "KEYEVENT_STEM_2": "266",
                "KEYEVENT_STEM_3": "267",
                "KEYEVENT_DPAD_UP_LEFT": "268",
                "KEYEVENT_DPAD_DOWN_LEFT": "269",
                "KEYEVENT_DPAD_UP_RIGHT": "270",
                "KEYEVENT_DPAD_DOWN_RIGHT": "271",
                "KEYEVENT_MEDIA_SKIP_FORWARD": "272",
                "KEYEVENT_MEDIA_SKIP_BACKWARD": "273",
                "KEYEVENT_MEDIA_STEP_FORWARD": "274",
                "KEYEVENT_MEDIA_STEP_BACKWARD": "275",
                "KEYEVENT_SOFT_SLEEP": "276",
                "KEYEVENT_CUT": "277",
                "KEYEVENT_COPY": "278",
                "KEYEVENT_PASTE": "279",
                "KEYEVENT_SYSTEM_NAVIGATION_UP": "280",
                "KEYEVENT_SYSTEM_NAVIGATION_DOWN": "281",
                "KEYEVENT_SYSTEM_NAVIGATION_LEFT": "282",
                "KEYEVENT_SYSTEM_NAVIGATION_RIGHT": "283",
                "KEYEVENT_KEYCODE_ALL_APPS": "284",
                "KEYEVENT_KEYCODE_REFRESH": "285",
                "KEYEVENT_KEYCODE_THUMBS_UP": "286",
                "KEYEVENT_KEYCODE_THUMBS_DOWN": "287"
        }
        if char in char_list.keys():
            return char_list[char]
        return None
    def get_array_of_char(self,string):
        ''' get_array_of_char '''
        array_of_key_event=[]
        if len(string) > 1 and self.key_event_table(string) is not None:
            array_of_key_event.append(self.key_event_table(string.upper()))
        else:
            for letter in list(string):
            #for letter in [char for char in string]:
                key_value = self.key_event_table('KEYEVENT_' + letter.upper())
                if key_value is None:
                    print ('this char: ' + letter + ' will be ignored')
                else:
                    if letter.isupper() is True:
                        array_of_key_event.append(
                            key_value + "," + self.key_event_table('KEYEVENT_META_SHIFT_ON'))
                    else:
                        array_of_key_event.append(key_value)
        return array_of_key_event



from behave import *


@given('we have behave installed')
def step_impl(context):
    pass


@when('we implement a test')
def step_impl(context):
    assert True is not False


@then('behave will test it for us!')
def step_impl(context):
    assert context.failed is False



''' Bcolors '''
HEADER = "\033[95m"
NABLUE = "\033[94m"
OKGREEN = "\033[92m"
WARNING = "\033[93m"
FAIL = "\033[91m"
ENDC = "\033[0m"
BOLD = "\033[1m"
UNDERLINE = "\033[4m"
class InitPrinter:
    ''' InitPrinter '''
    @staticmethod
    def underline(test_name):
        '''underline'''
        print(BOLD + UNDERLINE + "Step: " + test_name +
              ENDC)
    @staticmethod
    def success(ok_string):
        '''success'''
        print(OKGREEN + ok_string + ENDC)
    @staticmethod
    def danger(ko_string):
        '''danger'''
        print(FAIL + ko_string + ENDC)
    @staticmethod
    def warning(na_string):
        '''warning'''
        print(NABLUE + na_string + ENDC)
    @staticmethod
    def print_important_text(txt_string):
        '''print_important_text'''
        print(BOLD + txt_string + ENDC)



"""System module."""
from __future__ import absolute_import
import sys
import os
import json
import collections
from pathlib import Path
import base64
import requests
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
from PIL import Image


class TypeDir:
    ''' Type Dir '''
    PROJECT_MAIN_DIR = 0
    PROJECT_DIR = 1
    IDCYCLE_DIR = 2
    STEP_DIR = 3
    CONFIGURATIONSTEP_DIR = 4
    PLUGIN_DIR = 5
    ENVIRONMENTS_DIR = 6


class Connection:
    ''' Connection '''
    @staticmethod
    def start(method, url, payload=None, api_key=None, debug=False):
        ''' start '''
        req = None
        headers = {"Content-Type": "application/json", "Idelium-Key": api_key}
        if method == "POST":
            req = requests.post(url,
                              headers=headers,
                              data=json.dumps(payload),
                              verify=False)
        elif method == "PUT":
            req = requests.put(url,
                             headers=headers,
                             data=json.dumps(payload),
                             verify=False)
        elif method == "GET":
            req = requests.get(url, headers=headers, verify=False)
        if debug is True:
            print("Response: " + req.text)
            print("Headers: " + json.dumps(headers))
            print("Payload: " + json.dumps(payload))
            print(str(req.status_code) + " " + method + " " + url)
        return json.loads(req.text, object_pairs_hook=collections.OrderedDict)


class IdeliumWs:
    ''' IdeliumWs'''
    @staticmethod
    def create_folder(config):
        ''' create folder '''
        url = config["api_idelium"] + "testcycle"
        payload = {
            "testCycleId": config['idCycle'],
        }
        return Connection.start("POST", url, payload, config["ideliumKey"],
                                config["is_debug"])

    @staticmethod
    def create_test(config, id_test, name):
        ''' create test '''
        url = config["api_idelium"] + "test"
        payload = {
            "testCycleId": config['idCycle'],
            "testId": id_test,
            "name": name,
        }
        return Connection.start("POST", url, payload, config["ideliumKey"],
                                config["is_debug"])

    @staticmethod
    def update_test(config, id_test, status):
        ''' create test '''
        url = config["api_idelium"] + "test"
        payload = {
            "testId": id_test,
            "status": status,
        }
        return Connection.start("PUT", url, payload, config["ideliumKey"],
                                config["is_debug"])

    @staticmethod
    def create_step(config, id_test, id_step, name, status, data, typeofstep):
        ''' create step '''
        url = config["api_idelium"] + "step"
        payload = {
            "testCycleId": config['idCycle'],
            "testId": id_test,
            "stepId": id_step,
            "name": name,
            "status": int(status),
            "data": json.dumps(data),
            "type": typeofstep,
            "screenshots": "[]",
        }
        return Connection.start("POST", url, payload, config["ideliumKey"],
                                config["is_debug"])

    @staticmethod
    def update_step(config, id_step, screenshots):
        ''' update step '''
        url = config["api_idelium"] + "step"
        payload = {
            "stepId": id_step,
            "screenshots": json.dumps(screenshots),
        }
        return Connection.start("PUT", url, payload, config["ideliumKey"],
                                config["is_debug"])

    @staticmethod
    def get_environments(config):
        ''' get environment '''
        url = config["api_idelium"] + "environments/" + str(
            config["idProject"])
        return Connection.start("GET", url, None, config["ideliumKey"],
                                config["is_debug"])

    @staticmethod
    def get_cycles(config):
        ''' get cycles '''
        url = config["api_idelium"] + "testcycle/" + config['idCycle']
        json_cycle = Connection.start("GET", url, None, config["ideliumKey"],
                                     config["is_debug"])
        if "config" in json_cycle:
            return json.loads(json_cycle["config"])
        return -1

    @staticmethod
    def get_tests(config, id_test):
        ''' get tests '''
        url = config["api_idelium"] + "test/" + str(id_test)
        json_test = Connection.start("GET", url, None, config["ideliumKey"],
                                    config["is_debug"])
        return json.loads(json_test["config"])

    @staticmethod
    def get_step(config, id_step):
        ''' get step '''
        url = config["api_idelium"] + "step/" + str(id_step)
        json_step = Connection.start("GET", url, None, config["ideliumKey"],
                                    config["is_debug"])
        return {
            "objectStep": json.loads(json_step["config"]),
            "step_json_name": json_step["name"] + "_" + str(id_step),
            "step_json_description": json_step["name"],
        }
    @staticmethod
    def create_directories(config):
        ''' create directories '''
        configuration_directories = [
            config["dir_idelium_scripts"],
            config["dir_idelium_scripts"] + "/" + config["idProject"],
            config["dir_idelium_scripts"] + "/" + config["idProject"] + "/" +
            config['idCycle'],
            config["dir_idelium_scripts"] + "/" + config["idProject"] + "/" +
            config['idCycle'] + "/step",
            config["dir_idelium_scripts"] + "/" + config["idProject"] + "/" +
            config['idCycle'] + "/configurationStep",
            config["dir_idelium_scripts"] + "/" + config["idProject"] + "/" +
            config['idCycle'] + "/plugin",
            config["dir_idelium_scripts"] + "/" + config["idProject"] + "/" +
            config['idCycle'] + "/environments",
        ]
        print("start download configuration")
        return configuration_directories

    def download_configuration_files(self, config):
        ''' download configuration files '''
        printer = config["printer"]
        configuration_step = {}
        configuration_directories = self.create_directories(config)
        object_cycle = self.get_cycles(config)
        if object_cycle == -1:
            printer.danger("The id_cycle " + str(config["idCycle"]) +
                           " not exist")
            if config['ideliumServer'] is False:
                sys.exit(1)
            else:
                return False
        array_steps = {}
        array_environments = {}
        array_plugins = {}
        config_step = None
        #search cycle for this cycle
        for cycle in object_cycle:
            object_test = self.get_tests(config, cycle["id"])
            for test in object_test:
                step = self.get_step(config, test["id"])
                # write step
                array_steps[step["step_json_name"]] = step["objectStep"]
                print(step["step_json_name"])
                json_file_path = (configuration_directories[TypeDir.STEP_DIR] +
                                "/" + step["step_json_name"] + ".json")
                if config["local"] is True and (
                        Path(json_file_path).exists() is False
                        or config["forcedownload"] is True):
                    with open(json_file_path, "w") as file:
                        json.dump(step["objectStep"],
                                  file,
                                  indent=4,
                                  sort_keys=False)
        #write_configuration_step
        config_step = None
        json_file_path = (
            configuration_directories[TypeDir.CONFIGURATIONSTEP_DIR] +
            "/config_step.json")
        if config["local"] is True and (Path(json_file_path).exists() is False
                                        or config["forcedownload"] is True):
            with open(json_file_path, "w") as file:
                json.dump(configuration_step, file, indent=4, sort_keys=False)
        #search  plugins for projectId
        url = config["api_idelium"] + "plugins/" + str(config["idProject"])
        json_plugins = Connection.start("GET", url, None, config["ideliumKey"],
                                       config["is_debug"])
        for plugin_det in json_plugins:
            url = config["api_idelium"] + "plugin/" + str(plugin_det["id"])
            json_plugin = Connection.start("GET", url, None,
                                          config["ideliumKey"],
                                          config["is_debug"])
            #save  plugin for projectId
            array_plugins[json_plugin["name"]] = json_plugin["code"]
            py_file_path = (configuration_directories[TypeDir.PLUGIN_DIR] + "/" +
                          json_plugin["name"] + ".py")
            plugins_dir = config["dir_idelium_scripts"] + "/plugin"
            if Path(plugins_dir).exists() is False:
                os.mkdir(plugins_dir)
                if config["is_debug"] is True:
                    print("created temporary directory", plugins_dir)
            py_file_path = plugins_dir + "/" + json_plugin["name"] + ".py"

            if Path(py_file_path).exists(
            ) is False or config["forcedownload"] is True:
                py_file = open(py_file_path, "wt")
                py_file.write(json_plugin["code"])
                py_file.close()
        #download environments
        json_environments = self.get_environments(config)
        printer.success("finish download file")
        for env in json_environments:
            url = config["api_idelium"] + "environment/" + str(env["id"])
            json_environment = Connection.start("GET", url, None,
                                               config["ideliumKey"],
                                               config["is_debug"])
            file_name_env = json_environment["code"]
            code_environment = json.loads(
                json_environment["config"],
                object_pairs_hook=collections.OrderedDict)
            array_environments[file_name_env] = code_environment
            json_file_path = (
                configuration_directories[TypeDir.ENVIRONMENTS_DIR] + "/" +
                file_name_env + ".json")
            if config["local"] is True and (
                    Path(json_file_path).exists() is False
                    or config["forcedownload"] is True):
                with open(json_file_path, "w") as file:
                    json.dump(code_environment, file, indent=4, sort_keys=False)
        return {
            "steps":
            array_steps,
            "environments":
            array_environments,
            "plugins":
            array_plugins,
            "configStep":
            config_step,
            "environmentDir":
            configuration_directories[TypeDir.ENVIRONMENTS_DIR],
            "stepDir":
            configuration_directories[TypeDir.STEP_DIR],
            "config_stepDir":
            configuration_directories[TypeDir.CONFIGURATIONSTEP_DIR],
            "id_cycleDir":
            configuration_directories[TypeDir.IDCYCLE_DIR],
        }

    def start_test(self, idelium, test_configurations, config):
        ''' start test '''
        if config['ideliumServer'] is True:
            Path(config['dir_idelium_scripts'] + 'server').touch()
        wrapper = idelium.get_wrapper(config)
        object_cycle = self.get_cycles(config)
        driver = None
        id_cycle = self.create_folder(config)['idCycle']
        for cycle in object_cycle:
            #search test for this cycle
            printer = config["printer"]
            object_test = self.get_tests(config, cycle["id"])
            printer.success("Test: " + cycle["description"])
            id_test = self.create_test(config, id_cycle, cycle["name"])['idTest']
            test_failed = False
            for test in object_test:
                if test_failed is False:
                    json_step = test_configurations["steps"][test["name"] +
                                                            "_" +
                                                            str(test["id"])]
                    printer.underline(json_step["name"] + "(" +
                                      str(test["id"]) + ")")
                    config["wrapper"] = wrapper
                    config["printer"] = printer
                    config["json_step"] = json_step
                    object_return = idelium.execute_step(driver, config)
                    status = object_return["status"]
                    driver = object_return["driver"]
                    postman_data=object_return["postman_data"]
                    typeofstep=object_return["type"]
                    step_failed = object_return["step_failed"]
                    config["status"] = status
                    config["step_failed"] = step_failed
                    id_step = None
                    #test["name"],
                    if config["test"] is False:
                        id_step = self.create_step(config, id_test, test["id"],
                                                 json_step["name"],
                                                 status,
                                                 postman_data,
                                                 typeofstep
                                                 )['idStep']
                    if status in ('2','5') and object_return['type']=='seleniumOrAppium':
                        path = "screenshots/"
                        file_name = str(id_test) + ".png"
                        if not os.path.exists(path):
                            os.makedirs(path)
                        if config["json_step"]["attachScreenshot"] is True:
                            wrapper.screen_shot(driver, path + file_name,config['ideliumServer'])
                        if config["test"] is False:
                            file_name_jpg = path + str(id_test) + ".jpg"
                            with Image.open(path + file_name) as img:
                                rgb_im = img.convert("RGB")
                                rgb_im.save(file_name_jpg)
                                with open(file_name_jpg, "rb") as img_file:
                                    screenshot_base64 = base64.b64encode(
                                        img_file.read())
                                    self.update_step(
                                        config,
                                        id_step,
                                        [
                                            "data:image/jpg;base64," +
                                            str(screenshot_base64)[2:-1]
                                        ],
                                    )

                            os.unlink(path + file_name)
                            os.unlink(file_name_jpg)
                        if config["json_step"]["failedExit"] is True:
                            printer.danger(
                            "The test '" + cycle["name"] +
                            "' it is forcibly interrupted due to the blocking failure of the step"
                            )
                            id_test = self.update_test(config, id_test, 2)
                            test_failed = True
                    else:
                        self.update_test(config, id_test, 1)
            if config['ideliumServer'] is True:
                os.remove(config['dir_idelium_scripts'] + 'server')
            if driver != None:
                driver.quit()



"""System module."""
from __future__ import absolute_import
from selenium.webdriver.common.by import By

class SelBy() :
    ''' SelBy '''
    @staticmethod
    def get_by (key):
        ''' get_by'''
        keys_array= {
            'ID': By.ID,
            'XPATH': By.XPATH,
            'LINKTEXT': By.LINK_TEXT,
            'LINK_TEXT': By.LINK_TEXT,
            'PARTIAL_LINK_TEXT': By.PARTIAL_LINK_TEXT,
            'NAME' : By.NAME,
            'TAG_NAME' : By.TAG_NAME,
            'TAG' : By.CLASS_NAME,
            'CLASS_NAME' : By.CLASS_NAME,
            'CLASS' : By.CLASS_NAME,
            'CSS_SELECTOR' : By.CSS_SELECTOR,
            'CSS' : By.CSS_SELECTOR,
        }
        if key.upper() in keys_array.keys():
            return keys_array[key.upper()]
        return None



"""System module."""
from __future__ import absolute_import
import sys
import tempfile
from pathlib import Path
import selenium


class InitIdelium():
    ''' init '''
    @staticmethod
    def get_selenium_version():
        ''' version selenium '''
        return selenium.__version__

    @staticmethod
    def get_syntax():
        ''' help command line '''
        return """
    \033[1mUsage\033[0m: idelium [options]

    Options:

    --help                  show this help
    --idCycle               cycle id to associate to the execution "idCycle1,idCycle2,...."
    --idProject             force idProject
    --environment           environment json config file (required)
    --useragent             set useragent for the test
    --test                  for testing without store the results
    --verbose               for debugging 
    --dirChromedriver       default path of chromedriver path ("./chromedriver/last")
    --dirConfigurationStep  default path ("./configurationStep") for configuration steps 
    --dirStepFiles          default path ("./step") of directory for step files 
    --dirIdeliumScript      default path (".") of directory for step files
    --width                 default width of screen 1024
    --height                default height of screen 768
    --device                if is set useragent,height and width are ignored
    --url                   url for test 
    --ideliumwsBaseurl      idelium server url ex: https://localhost
    --reportingService      where the data will be save: idelium | zephyr
    --ideliumKey            is the key for access to the idelium api
    --forcedownload         force to ovewrite the configuration files
    --idChannel             idChannel
    
    Idelium server
    --ideliumServer         with this option idelium-cli is in server mode
    --ideliumServerPort     default is 8691

    Zephir 
    --jiraApiUrl            for change the default jira url (https://<host jira>/rest/api/latest/)
    --idJira                jira id (required if idVersion and idCycle not setted)
    --idVersion             version id to associate the execution 
    --username              jira username (required)
    --password              jira password (required)



    For Example: 

    default reporting service: idelium --ideliumKey=1234 --idCycle=2 --idProject=8 --environment=prod

    working with jira/zephyr: idelium --reportingService=zephyr --idJira=prj-1234 --username=user --password=secret --environment=prod.json --useragent='apple 1134'

    """
    @staticmethod
    def get_reguired_parameters():
        ''' list of required parameters '''
        return {
            "idProject" : 0,
            "idCycle" : 0,
            "environment" : 0,
            "ideliumKey": 0,
        }
    @staticmethod
    def get_default_parameters():
        ''' set default parameters '''
        return {
            'execution_name': 'automation test python',
            'reportingService': 'idelium',
            'ideliumwsBaseurl': 'https://service.idelium.io',
            'base_url':None,
            'zephyrApiUrl':None,
            'jiraApiUrl':None,
            'dir_plugins':"plugin",
            'test':False,
            'is_debug':False,
            'device':None,
            'width': 1920,
            'height':1080,
            'username':None,
            'password':None,
            'environment':None,
            'idJira':None,
            'fileSteps':None,
            'idVersion':None,
            'idCycle':None,
            'useragent':None,
            'idProject':None,
            'idChannel' : None,
            'url' : None,
            'isRealDevice' : False,
            'os' : None,
            'appiumServer' : None,
            'appiumDesiredCaps' : None,
            'count':0,
            'ideliumKey':None,
            'forcedownload':False,
            'local':False,
            'ideliumServer': False,
            'ideliumServerPort': 8691,
        }
    def define_parameters(self,args,ideliumws,printer):
        ''' set all necessary parameters '''
        cl_params= self.get_default_parameters()
        check_required=self.get_reguired_parameters()
        cl_params['dir_idelium_scripts'] = tempfile.gettempdir()
        count=0
        for i in args:
            array_command=i.split("=")
            command=array_command[0][2:]
            if command in cl_params:
                if command == 'ideliumKey':
                    cl_params['ideliumKey']=''
                    if len(array_command)==3:
                        cl_params['ideliumKey'] = array_command[1] + '=' 
                    else:
                        cl_params['ideliumKey'] = array_command[1]
                elif command == 'ideliumServer':
                    cl_params['ideliumServer'] = True
                elif command == 'ideliumServerPort': 
                    cl_params['ideliumServerPort'] = int(array_command[1])
                else:
                    cl_params[command]=array_command[1]
                if command in check_required:
                    check_required[command]=1
            elif array_command[0] == "--verbose":
                cl_params['is_debug'] = True
            elif array_command[0] == "--help":
                print(self.get_syntax())
                sys.exit(0)
            else:
                if count > 0:
                    print (self.get_syntax())
                    print("\n" + array_command[0] + ": is not a valid option")
                    sys.exit(1)
            count=count=count + 1
        count_req=0
        for i in check_required:
            count_req=count_req + check_required[i]
        if cl_params['ideliumServer'] is False:
            if cl_params['ideliumKey']is None:
                file_idelium_key=str(Path.home()) + '/.idelium'
                if Path(file_idelium_key).is_file() is True:
                    file = open(file_idelium_key, "r")
                    cl_params['ideliumKey']=file.read()
                else:
                    print (self.get_syntax())
                    printer.danger('ideliumKey is not setted !')
                    sys.exit(1)
        if cl_params['ideliumServer'] is False:
            if cl_params['reportingService'] == 'idelium':
                if (cl_params['idProject'] is None
                        or cl_params['idCycle'] is None):
                    print(self.get_syntax())
                    printer.danger("\nidProject and idCycle are mandatory")
                    sys.exit(1)
                if cl_params['reportingService'] == 'zephyr':
                    if count_req < 4:
                        print(self.get_syntax())
                        printer.danger("\nMissed required options")
                        sys.exit(1)
                if cl_params['environment'] is None:
                    print(self.get_syntax())
                    printer.danger("\nenvironment must be set")
                    sys.exit(1)
        return {
            'cl_params': cl_params,
        }

    def load_parameters(self, cl_params, ideliumws, printer):

        sys.path.append(cl_params['dir_idelium_scripts'])

        cl_params['api_idelium']=cl_params['ideliumwsBaseurl'] + '/api/ideliumcl/'
        cl_params['printer']=printer
        test_config=None
        json_config=None
        json_step_config=None
        test_config = ideliumws.download_configuration_files(cl_params)
        if test_config is False:
            return False
        print('Environment:' + cl_params['environment'])
        if cl_params['environment'] in test_config['environments']:
            json_config=test_config['environments'][cl_params['environment']]
        else:
            printer.danger('Environment "' + cl_params['environment']
                            + '" or idProject ' + cl_params['idProject'] + ' not exist')
            sys.exit(1)
        if 'userAgent' in json_config:
            cl_params['user_agent']=json_config['userAgent']
        if 'isRealDevice' in json_config:
            cl_params['isRealDevice']=json_config['isRealDevice']
        if 'appiumServer' in json_config:
            cl_params['appiumServer']=json_config['appiumServer']
        if 'isRealDevice' in json_config:
            cl_params['appiumDesiredCaps']=json_config['appiumDesiredCaps']
        if cl_params['idProject'] is not None:
            json_config['projectId'] = cl_params['idProject']

        json_step_config=test_config['configStep']
        if cl_params['idProject'] is not None and json_step_config is not None:
            json_step_config['idProject']=cl_params['idProject']
        if 'device' in json_config:
            cl_params['device'] = json_config['device']
        if cl_params['url'] is not None:
            json_config['url']=cl_params['url']
        cl_params['json_config']=json_config
        cl_params['json_step_config']=json_step_config

        return {
            'cl_params': cl_params,
            'test_config': test_config
        }
        



from __future__ import absolute_import
from argparse import HelpFormatter

import json
import collections
from re import A
import sys
import datetime
import time
import os
from urllib.parse import urlencode
import requests
from idelium._internal.commons.ideliumprinter import InitPrinter
'''from postpy2.core import PostPython'''
from idelium._internal.thirdparties.postpy2.core import PostPython


class PostmanCollection:
    ''' PostmanCollection '''
    @staticmethod
    def start_postman_test(collection):
        print ('postman')
        printer = InitPrinter()
        runner = PostPython(collection['collection'])
        if collection['environment'] != None:
            runner.environments.load(collection['environment'])
        #response = runner.AuthDigest.digestauth_request()
        folders= runner.getmethods()
        print ("---------")
        response_test={}
        for methods in  folders:
            for method in methods:
                array_name=method.split(".")
                call=array_name[1] + "."  + array_name[2]
                object = getattr(runner, array_name[1])
                response = getattr(object, array_name[2])
                #try: 
                test=response()
                attributes = [
                            'apparent_encoding',
                            'content',
                            'cookies',
                            'elapsed',
                            'encoding',
                            'headers',
                            'history',
                            'is_permanent_redirect',
                            'is_redirect',
                            'links',
                            'next',
                            'ok',
                            'raw',                 
                            'reason',
                            'request',
                            'status_code',
                            'text',
                            'url'                        
                    ]
                results={}
                for attribute in attributes:
                    results[attribute]=str(getattr(test,attribute))
                response_test[call] = results
                #except:
                #    printer.danger ("Error call: " + call  +  "()")
                #print ("------------------------------------------------")
                
        #print(json.dumps(response_test,indent=4))
        with open('data.json', 'w', encoding='utf-8') as f:
            json.dump(response_test,f, indent=4)
        return response_test
        



"""System module."""
from __future__ import absolute_import
import json
import collections
import sys
import datetime
import time
import os
from urllib.parse import urlencode
import requests
from idelium._internal.commons.ideliumprinter import InitPrinter

class ZephyrConnection:
    ''' ZephyrConnection '''
    @staticmethod
    def get_date():
        ''' Get the date '''
        today = datetime.date.today()
        return today.strftime("%Y-%m-%d")
    @staticmethod
    def get_test_case(is_debug, is_test, api_url, zapi_url, issue,
                    username, password):
        ''' Get the test case '''
        os.environ["NO_PROXY"] = "<host>"
        printer = InitPrinter()
        is_ok = False
        #give all the step of testcase
        url_issue = api_url + "issue/" + issue
        req = requests.get(url_issue, auth=(username, password))
        if is_debug is True:
            print(str(req.status_code) + " GET " + url_issue)
            print(req.text)
        if req.status_code != 200:
            printer.danger("credential error o issue jira not exist")
            sys.exit(1)
        json_jira = json.loads(req.text,
                               object_pairs_hook=collections.OrderedDict)
        if "status" in json_jira["fields"]:
            if "name" in json_jira["fields"]["status"]:
                if (json_jira["fields"]["status"]["name"] == "Executable"
                        or is_test is True):
                    is_ok = True
                else:
                    printer.danger("The issue " + issue +
                                   " is not 'Executable' but '" +
                                   json_jira["fields"]["status"]["name"] +
                                   "' so is skipped")
        urlzephyr = zapi_url + "teststep/" + json_jira["id"]
        req = requests.get(urlzephyr, auth=(username, password))
        if is_debug is True:
            print(str(req.status_code) + " GET " + urlzephyr)
            print("Response: " + req.text)
        json_zephyr = json.loads(req.text,
                                 object_pairs_hook=collections.OrderedDict)
        return {
            'zapyhrObject': json_zephyr,
            "jiraIssueId": json_jira["id"],
            "isGoodIssueForTest": is_ok,
        }
    def create_cycle(self, is_debug, zapi_url, project_id, version_id, name,
                    username, password):
        ''' create_cycle '''
        os.environ["NO_PROXY"] = "<host>"
        url = zapi_url + "cycle"
        if version_id is None:
            version_id = "-1"
        headers = {"Content-Type": "application/json"}
        payload = {
            "name": name,
            "startDate": self.get_date(),
            "endDate": self.get_date(),
            "projectId": project_id,
            "versionId": version_id,
            "sprintId": None,
        }
        req = requests.post(url,
                          headers=headers,
                          data=json.dumps(payload),
                          auth=(username, password))
        if is_debug is True:
            print(str(req.status_code) + " POST " + url)
            print("Payload: " + json.dumps(payload))
            print("Response: " + req.text)
        return json.loads(req.text)
    @staticmethod
    def create_execution(
        is_debug,
        zapi_url,
        cycle_id,
        folder_id,
        issue_id,
        project_id,
        username,
        password,
    ):
        ''' create execution '''
        os.environ["NO_PROXY"] = "<host>"
        url = zapi_url + "execution"
        headers = {"Content-Type": "application/json"}
        payload = {
            "cycleId": cycle_id,
            "issueId": issue_id,
            "projectId": project_id,
        }
        if folder_id is not None:
            payload["folderId"] = folder_id
        req = requests.post(url,
                          headers=headers,
                          data=json.dumps(payload),
                          auth=(username, password))
        if is_debug is True:
            print(str(req.status_code) + " POST " + url)
            print("Payload: " + json.dumps(payload))
            print("Response: " + req.text)
        return json.loads(req.text)

    def create_cycle_folder(
        self,
        is_debug,
        environment,
        zapi_url,
        cycle_id,
        version_id,
        project_id,
        username,
        password,
    ):
        ''' create folder cycle '''
        os.environ["NO_PROXY"] = "<host>"
        url = zapi_url + "folder/create"
        headers = {"Content-Type": "application/json"}
        name_folder = ("[" + environment + "] " +
                      datetime.datetime.fromtimestamp(
                          time.time()).strftime("%Y-%m-%d %H:%M:%S"))
        payload = {
            "cycleId": cycle_id,
            "name": name_folder,
            "description": "created test folder for this cycle",
            "projectId": project_id,
            "versionId": version_id,
        }
        req = requests.post(url,
                          headers=headers,
                          data=json.dumps(payload),
                          auth=(username, password))
        if is_debug is True:
            print(str(req.status_code) + " POST " + url)
            print("Payload: " + json.dumps(payload))
            print("Response: " + req.text)
        return json.loads(req.text)
    @staticmethod
    def get_executions(
        is_debug,
        zapi_url,
        cycle_id,
        version_id,
        project_id,
        offset,
        username,
        password,
    ):
        ''' get_executions '''
        os.environ["NO_PROXY"] = "<host>"
        url = zapi_url + "execution"
        headers = {"Content-Type": "application/json"}
        payload = {
            "cycleId": cycle_id,
            "versionId": version_id,
            "action": "expand",
            "projectId": project_id,
            "offset": offset,
            "sorter": "OrderId:ASC",
        }
        query_string = urlencode(payload)
        url = url + "?" + query_string
        req = requests.get(url, headers=headers, auth=(username, password))
        if is_debug is True:
            print(str(req.status_code) + " GET " + url)
            print("Payload: " + json.dumps(payload))
            print("Response: " + req.text)
        return json.loads(req.text)
    @staticmethod
    def get_step_id(is_debug, zapi_url, execution_id, username, password):
        '''get_step_id'''
        os.environ["NO_PROXY"] = "<host>"
        url = zapi_url + "stepResult?execution_id=" + str(execution_id)
        req = requests.get(url, auth=(username, password))
        if is_debug is True:
            print(str(req.status_code) + " GET " + url)
            print("Response: " + req.text)
        return json.loads(req.text)
    @staticmethod
    def update_test_step(is_debug, zapi_url, step_id, status, step_failed,
                       username, password):
        ''' get test step'''
        os.environ["NO_PROXY"] = "<host>"
        url = zapi_url + "stepResult/" + str(step_id)
        headers = {"Content-Type": "application/json"}
        comment = ""
        if step_failed is not None:
            if "note" in step_failed:
                comment = "Error:" + step_failed["note"]
        payload = {"status": status, "comment": comment}
        req = requests.put(url,
                         headers=headers,
                         data=json.dumps(payload),
                         auth=(username, password))
        if is_debug is True:
            print(str(req.status_code) + " PUT " + url)
            print("Payload: " + json.dumps(payload))
            print("Response: " + req.text)
        return json.loads(req.text)
    @staticmethod
    def update_execution(is_debug, zapi_url, execution_id, status,
                        username, password):
        ''' Update execution'''
        os.environ["NO_PROXY"] = "<host>"
        url = zapi_url + "execution/" + str(execution_id) + "/execute"
        headers = {"Content-Type": "application/json"}
        payload = {
            "status": status,
        }
        req = requests.put(url,
                         headers=headers,
                         data=json.dumps(payload),
                         auth=(username, password))
        if is_debug is True:
            print(str(req.status_code) + " PUT " + url)
            print("Payload: " + json.dumps(payload))
            print("Response: " + req.text)
        return json.loads(req.text)
    @staticmethod
    def add_attachment_buffered(
        is_debug,
        zapi_url,
        path,
        file_name,
        step_id,
        entity_type,
        username,
        password,
    ):
        '''Add attachment buff'''
        os.environ["NO_PROXY"] = "<host>"
        #Possible types: EXECUTION, STEPRESULT
        url = (zapi_url + "attachment?entityId=" + str(step_id) +
               "&entityType=" + entity_type)
        file = open(path + file_name, "rb")
        files = {"file": (file_name, file, "multipart/form-data")}
        headers = {
            "X-Atlassian-Token": "nocheck",
            "Accept": "application/json",
        }
        req = requests.post(url,
                          headers=headers,
                          files=files,
                          auth=(username, password))
        if is_debug is True:
            print(str(req.status_code) + " POST " + url)
            print("Response: " + req.text)
        return json.loads(req.text)

    #
    #  Jira/Zaphyr interface
    #
    def go_execution(self, config):
        '''Execute'''
        printer = config["printer"]
        project_id = config["json_config"]["projectId"]
        for id_cycle in config['idCycle'].split(","):
            printer.success("Start cycleId:" + str(id_cycle))
            exit = False
            count = 0
            offset = 0
            folder_id = None

            if config["is_test"] is False:
                json_create_folder = self.create_cycle_folder(
                    config["is_debug"],
                    config["json_config"]["environment"],
                    config["zephyrApiUrl"],
                    id_cycle,
                    config["idVersion"],
                    config["project_id"],
                    config["username"],
                    config["password"],
                )
                if "id" in json_create_folder:
                    folder_id = json_create_folder["id"]
                else:
                    printer.danger("Jira Error:" + json_create_folder["error"])
                    sys.exit()
            while exit is False:
                return_execution = self.get_executions(
                    config["is_debug"],
                    config["zephyrApiUrl"],
                    id_cycle,
                    config["idVersion"],
                    project_id,
                    offset,
                    config["username"],
                    config["password"],
                )
                for execution in return_execution["executions"]:
                    id_jira = execution["issueKey"]
                    original_execution_id = execution["id"]
                    execution_name = execution["cycleName"]
                    id_cycle = execution["cycleId"]
                    printer.print_important_text(id_jira + ": " +
                                                 execution["summary"])
                    config["execution_name"] = execution_name
                    config["id_jira"] = id_jira
                    config["original_execution_id"] = original_execution_id
                    config["folder_id"] = folder_id
                    config['idCycle'] = id_cycle
                    self.start_test_case(config)
                    count = count + 1
                if count == return_execution["totalExecutions"]:
                    exit = True
                else:
                    offset = offset + 10

    def start_test_case(self, idelium, test_configurations, config):
        ''' start test case'''
        printer = config["printer"]
        wrapper = idelium.getWrapper(config)
        #project_id = config["json_config"]["projectId"]
        zapyhr_object = self.get_test_case(
            config["is_debug"],
            config["is_test"],
            config["jiraApiUrl"],
            config["zephyrApiUrl"],
            config["idJira"],
            config["username"],
            config["password"],
        )
        if (len(zapyhr_object['zapyhrObject'])
                and zapyhr_object["isGoodIssueForTest"] is True):
            execution_id = None
            return_execution = None
            if config["is_test"] is False:
                if config['idCycle'] is None:
                    self.create_cycle(
                        config["is_debug"],
                        config["zephyrApiUrl"],
                        config["project_id"],
                        config["idVersion"],
                        config["execution_name"],
                        config["username"],
                        config["password"],
                    )
                    return_execution = self.create_execution(
                        config["is_debug"],
                        config["zephyrApiUrl"],
                        config["returnCycle"]["id"],
                        config["folder_id"],
                        config['zapyhrObject']["jiraIssueId"],
                        config["project_id"],
                        config["username"],
                        config["password"],
                    )
                else:
                    return_execution = self.create_execution(
                        config["is_debug"],
                        config["zephyrApiUrl"],
                        config['idCycle'],
                        config["folder_id"],
                        config['zapyhrObject']["jiraIssueId"],
                        config["project_id"],
                        config["username"],
                        config["password"],
                    )
                all_steps_execution = None
                for execution in return_execution:
                    execution_id = execution
                    all_steps_execution = self.get_step_id(
                        config["is_debug"],
                        config["zephyrApiUrl"],
                        config["execution"],
                        config["username"],
                        config["password"],
                    )
            driver = None
            index = 0
            execution_status = "1"
            stop_execute_steps = False
            for test_case in zapyhr_object['zapyhrObject']["stepBeanCollection"]:
                # if is_testis True:
                #   input("Test Mode Press Enter to continue...")
                step = test_case["step"].lower()
                if step not in config["json_step_config"]:
                    printer.danger("Warning, the  step: '" + step +
                                   "' is not defined")
                    sys.exit(1)
                try:
                    file_step = (config["dir_step_files"] + "/" +
                                 config["json_step_config"][step] + ".json")
                    with open(file_step) as json_data:
                        json_step = json.load(json_data)
                except BaseException as err:
                    printer.danger("Warning, the file step: " + file_step +
                                   " not exist or is not a json (err 1)")
                    print(err)
                    sys.exit(1)
                if stop_execute_steps is False:
                    printer.underline(config["json_step"]["name"] + " (" +
                                      str(test_case["id"]) + ")")
                    config["wrapper"] = wrapper
                    config["json_step"] = json_step
                    #object_return = self.execute_step(driver,
                    #                                 test_configurations,
                    #                                 config)
                    #status = object_return["status"]
                    #driver = object_return["driver"]
                    #step_failed = object_return['stepFailed']
                    step_failed = False
                    status = False
                    if config["is_test"] is False:
                        self.update_test_step(
                            config["is_debug"],
                            config["zephyrApiUrl"],
                            all_steps_execution[index]["id"],
                            status,
                            step_failed,
                            config["username"],
                            config["password"],
                        )
                    if status == "2" or status == "5":
                        execution_status = status
                        path = "screenshots/"
                        file_name = str(test_case["id"]) + ".png"
                        if not os.path.exists(path):
                            os.makedirs(path)
                        if config["json_step"]["attachScreenshot"] is True:
                            wrapper.screen_shot(driver, path + file_name,config['ideliumServer'])
                            if config["is_test"] is False:
                                self.add_attachment_buffered(
                                    config["is_debug"],
                                    config["zephyrApiUrl"],
                                    path,
                                    file_name,
                                    all_steps_execution[index]["id"],
                                    "STEPRESULT",
                                    config["username"],
                                    config["password"],
                                )
                            os.unlink(path + file_name)
                        if config["json_step"]["failedExit"] is True:
                            printer.danger(
                                "La issue " + config["idJira"] +
                                " e' forzamente interrotta causa fallimento bloccante dello step"
                            )
                            stop_execute_steps = True
                    index = index + 1
            driver.quit()
            if config["is_test"] is False:
                self.update_execution(
                    config["is_debug"],
                    config["zephyrApiUrl"],
                    execution_id,
                    execution_status,
                    config["username"],
                    config["password"],
                )
                if config["original_execution_id"] is not None:
                    self.update_execution(
                        config["is_debug"],
                        config["zephyrApiUrl"],
                        config["original_execution_id"],
                        execution_status,
                        config["username"],
                        config["password"],
                    )



"""System module."""
import os
import json
from http.server import BaseHTTPRequestHandler
from threading import Thread

class IdeliumServer(BaseHTTPRequestHandler):
    ''' IdeliumServer '''

    def init(idelium_get,params,ws,cllLib,prn):
        global idelium
        global cl_params
        global ideliumws
        global idelium_cl_lib
        global printer
        idelium = idelium_get
        cl_params = params
        ideliumws=ws
        idelium_cl_lib = cllLib
        printer = prn
    def do_GET(self):
        global idelium
        global cl_params
        global ideliumws
        global idelium_cl_lib
        global printer
        '''
            path= /idCycle/idProject/environment/<optional>
        '''
        data = self.path.split("/")
        if data[1] == 'reset':
            os.remove(cl_params['dir_idelium_scripts'] + 'server')
            self.wfile.write(
                bytes('{"message": "reset done"}', "utf-8"))
        else:
            print('ko')
            self.send_response(401)
            self.send_header("Content-type", "application/json")
            self.end_headers()
            self.wfile.write(
                    bytes('{"message": "invalid request"}', "utf-8"))

    def do_POST(self):
        # <--- Gets the size of data
        content_length = int(self.headers['Content-Length'])
        # <--- Gets the data itself
        post_data = self.rfile.read(content_length)
        if self.path == '/launchtest':
            json_post=json.loads(post_data.decode('utf-8'))
            self.send_response(200)
            self.send_header("Content-type", "application/json")
            self.end_headers()
            params = cl_params
            params['idCycle'] = json_post['idTestCycle']
            params['idProject'] = json_post['idProject']
            params['environment'] = json_post['environment']
            params['ideliumKey'] = json_post['key']
            params['browser'] = json_post['browser']
            self.close_connection
            if not os.path.exists(cl_params['dir_idelium_scripts'] + 'server'):
                define_parameters = idelium_cl_lib.load_parameters(
                    params, ideliumws, printer)
                if define_parameters is False:
                    self.wfile.write(
                        bytes('{"message": "bad parameters"}', "utf-8"))
                else:
                    params = define_parameters['cl_params']
                    params['json_config']['browser'] == params['browser']
                    test_config = define_parameters['test_config']
                    if cl_params['reportingService'] == 'idelium':
                        Thread(target=ideliumws.start_test, args=[
                            idelium, test_config, params]).start()
                        self.wfile.write(
                            bytes('{"message": "test started"}', "utf-8"))
            else:
                self.wfile.write(
                    bytes('{"message": "another test is running, please retry"}', "utf-8"))







import os
import sys
from os.path import abspath, dirname, join

from setuptools import find_packages, setup


this_dir = abspath(dirname(__file__))
with open(join(this_dir, 'README.md'), encoding='utf-8') as file:
    long_description = file.read()
def read(rel_path: str) -> str:
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, rel_path)) as fp:
        return fp.read()


def get_version(rel_path: str) -> str:
    for line in read(rel_path).splitlines():
        if line.startswith("IDELIUM_VERSION"):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    raise RuntimeError("Unable to find version string.")

setup(
    name='idelium',  # Required
    version=get_version('src/idelium/_internal/main.py'),
    description='Command line Test Automation Tool',  # Optional
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Idel Fuschini',  # Optional
    author_email='idel.fuschini@gmail.com',  # Optional
    license='MIT',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Build Tools",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
    ],
    url="https://idelium.io/",
    package_dir={"": "src"},

    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    package_data={
        "idelium": ["py.typed"],
    },
    packages=find_packages(
        where="src",
        exclude=["contrib", "docs", "tests*", "tasks"],
    ),

    # Specify which Python versions you support. In contrast to the
    # 'Programming Language' classifiers above, 'idelium install' will check this
    # and refuse to install the project if the version does not match. If you
    # do not support Python 2, you can simplify this to '>=3.5' or similar, see
    # https://packaging.python.org/guides/distributing-packages-using-setuptools/#python-requires

    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by idelium when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs idelium's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['selenium',
                      'libmagic',
                      'Appium-Python-Client',
                      'webdriver-manager',
                      'Pillow',
                      'Postpy2'
                      ],  # Optional

    # List additional groups of dependencies here (e.g. development
    # dependencies). Users will be able to install these using the "extras"
    # syntax, for example:
    #
    #   $ idelium install sampleproject[dev]
    #
    # Similar to `install_requires` above, these must be valid existing
    # projects.
    extras_require={  # Optional
        'dev': ['check-manifest'],
        'test': ['coverage'],
    },
    project_urls={  # Optional
        'Bug Reports': 'https://github.com/idelium/idelium-cli/issues',
        'Project': 'https://idelium.io',
        'Say Thanks!': 'http://www.idelfuschini.it',
        'Source': 'https://github.com/idelium/idelium-cli/',
    },
    entry_points={  # Optional
        'console_scripts': [
            "idelium=idelium._internal.main:main",
        ],
    },
    zip_safe=False,
    # NOTE: python_requires is duplicated in __idelium-runner__.py.
    # When changing this value, please change the other copy as well.
    python_requires=">=3.7",
)



"""System module."""
from __future__ import absolute_import
import time
import sys
from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from webdriver_manager.chrome import ChromeDriverManager
from webdriver_manager.firefox import GeckoDriverManager
from webdriver_manager.microsoft import EdgeChromiumDriverManager
from webdriver_manager.microsoft import IEDriverManager
from selenium.webdriver.chrome import service
from webdriver_manager.opera import OperaDriverManager
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.action_chains import ActionChains
from idelium._internal.commons.ideliumprinter import InitPrinter
from idelium._internal.commons.resultenum import Result
from idelium._internal.commons.seleniumkeyevent import EventKey
from idelium._internal.commons.seleniumby import SelBy


class IdeliumSelenium:
    ''' IdeliumSelenium '''
    @staticmethod
    def sleep(driver, config, object_step):
        ''' Sleep '''
        time.sleep(object_step["seconds"])
        return {'returnCode': Result.OK}

    def wait_and_click(self,driver, xpath_condition, note):
        ''' wait and click '''
        if self.wait_for_next_step_real(driver, xpath_condition,
                                        note) == Result.OK:
            if self.click_xpath(driver, xpath_condition, note) == Result.KO:
                return {'returnCode': Result.KO}
            else:
                return {'returnCode': Result.OK}
        else:
            return {'returnCode': Result.OK}

    @staticmethod
    def find_element_by_xpath(driver, xpath_condition, note=None):
        ''' find element by xpath condition '''
        return driver.find_element_by_xpath(xpath_condition)
    @staticmethod
    def find_elements_by_xpath(self, driver, xpath_condition, note=None):
        '''find elements by xpath condition'''
        return driver.find_elements_by_xpath(xpath_condition)
    @staticmethod
    def find_element(driver, by, target, note=None):
        ''' find element'''
        return driver.find_element(by, target)
    @staticmethod
    def find_elements(driver, by, target, note=None):
        ''' find elements'''
        return driver.find_elements(by, target)
    @staticmethod
    def page_source(driver, note=None):
        ''' page source for debug is useful '''
        return driver.page_source
    @staticmethod
    def switch_to_frame(driver, object_driver, note=None):
        ''' switch_to_frame '''
        driver.switch_to_frame(object_driver)
        return Result.OK
    @staticmethod
    def switch_to_default_content(driver, object, note=None):
        ''' switch_to_default_content '''
        driver.switch_to_default_content()
        return Result.OK
    @staticmethod
    def find_object_element(self, selenium_object, xpath_condition, note=None):
        ''' find_object_element '''
        return selenium_object.find_element_by_xpath(xpath_condition)
    @staticmethod
    def click_object(selenium_object, note):
        ''' click_object '''
        printer = InitPrinter()
        try:
            print(note, end="->", flush=True)
            time.sleep(1)
            selenium_object.click()
            printer.success("ok")
            return Result.OK
        except BaseException as err:
            printer.danger("FAILED")
            print(err)
            # sys.exit(1)
            return Result.KO
    @staticmethod
    def drag_and_drop(driver, config, object_step):
        ''' drag_and_drop '''
        printer = InitPrinter()
        try:
            drag_element = driver.find_element_by_xpath(
                object_step["xpathDrag"])
            drop_element = driver.find_element_by_xpath(
                object_step["xpathDrop"])
            action = ActionChains(driver)
            action.drag_and_drop(drag_element, drop_element).perform()
            return {'returnCode': Result.OK}
        except BaseException as err:
            printer.danger("FAILED")
            print(err)
            # sys.exit(1)
            return {'returnCode': Result.KO}
    def open_browser(self, driver, config, object_step):
        ''' open browser '''
        printer = InitPrinter()
        driver = None
        return_code = Result.OK
        ''' only for server mode '''
        if 'browser' in config:
           config["json_config"]["browser"] = config["browser"]
        if config["json_config"]["browser"] == "chrome":
            chrome_options = webdriver.ChromeOptions()
            if config["device"] is not None:
                mobile_emulation = {"deviceName": config["device"]}
                chrome_options = webdriver.ChromeOptions()
                chrome_options.add_experimental_option("mobileEmulation",
                                                       mobile_emulation)
            else:
                if config["useragent"] is not None:
                    chrome_options.add_argument("user-agent=" +
                                                config["useragent"])
            if "accept_self_certificate" in config["json_config"]:
                if config["json_config"]["accept_self_certificate"] is True:
                    chrome_options.add_argument("ignore-certificate-errors")
            try:
                driver = webdriver.Chrome(ChromeDriverManager().install())
            except BaseException as err:
                
                printer.warning("webdriver not found, try locally")
                try: 
                    driver=webdriver.Chrome()
                except:
                    printer.danger("webriver error")
                    if config['ideliumServer'] is False:
                        return_code = Result.KO
                        sys.exit(1)
        elif config["json_config"]["browser"] == "firefox":
            profile = webdriver.FirefoxProfile()
            if config["useragent"] is not None:
                profile.set_preference("general.useragent.override",
                                       config["useragent"])
            if "accept_self_certificate" in config["json_config"]:
                if config["json_config"]["accept_self_certificate"] is True:
                    profile.accept_untrusted_certs = True
            try:
                driver = webdriver.Firefox(GeckoDriverManager().install())
            except BaseException as err:
                printer.warning("webdriver not found, try locally")
                try:
                    driver = webdriver.Firefox()
                except:
                    printer.danger("webriver error")
                    if config['ideliumServer'] is False:
                        return_code = Result.KO
                        sys.exit(1)
        elif config["json_config"]["browser"] == "safari":
            try:
                driver = webdriver.Safari()
            except BaseException as err:
                printer.danger("webriver error")
                print(err)
                return_code = Result.KO
                if config['ideliumServer'] is False:
                    sys.exit(1)
        elif config["json_config"]["browser"] == "opera":
            try:
                webdriver_service = service.Service(OperaDriverManager().install())
                webdriver_service.start()
                driver = webdriver.Remote(webdriver_service.service_url, webdriver.DesiredCapabilities.OPERA)
            except BaseException as err:
                printer.danger("webriver error")
                print(err)
                return_code = Result.KO
                if config['ideliumServer'] is False:
                    sys.exit(1)
        elif config["json_config"]["browser"] == "edge":
            try:
                driver = webdriver.Edge(EdgeChromiumDriverManager().install())
            except BaseException as err:
                printer.warning("webdriver not found, try locally")
                try:
                    driver = webdriver.Edge()
                except:
                    printer.danger("webriver error")
                    if config['ideliumServer'] is False:
                        return_code = Result.KO
                        sys.exit(1)
        elif config["json_config"]["browser"] == "iexplorer":
            capabilities = webdriver.DesiredCapabilities().INTERNETEXPLORER
            if "accept_self_certificate" in config["json_config"]:
                if config["json_config"]["accept_self_certificate"] is True:
                    capabilities["acceptSslCerts"] = True
            try:
                driver=webdriver.Ie(IEDriverManager().install())
            except BaseException as err:
                printer.warning("webdriver not found, try locally")
                try:
                    driver = webdriver.Ie()
                except:
                    printer.danger("webriver error")
                    if config['ideliumServer'] is False:
                        return_code = Result.KO
                        sys.exit(1)
        else:
            printer.danger("driver not selected")
            if config['ideliumServer'] is False:
                sys.exit(1)
        if return_code == Result.OK:
            driver.set_window_size(config["width"], config["height"])
            if "url" in object_step:
                driver.get(object_step["url"])
            else:
                driver.get(config["json_config"]["url"])
            return_code = Result.OK
            object_step["xpath"] = config["json_config"]["xpath_check_url"]
            if object_step['xpath'] == '':
                object_step['xpath'] = '/html'
            if (self.wait_for_next_step(driver, config,
                                        object_step)['returnCode'] == Result.KO):
                return_code = Result.KO
                config["json_step"]["attachScreenshot"] = True
                config["json_step"]["failedExit"] = True
        return {"driver": driver, 'returnCode': return_code, "config": config}
    @staticmethod
    def write_localstorage(driver, config, object_step):
        ''' write_localstorage '''
        printer = InitPrinter()
        try:
            print(object_step["note"], end="->", flush=True)
            script_js = ""
            for object_data in object_step["dataLocalStorage"]:
                for key in object_data:
                    script_js = (script_js+ 'localStorage.setItem("' + key +
                                "\", '" + object_data[key] + "')\n")
            script_js = (
                script_js +
                "return Array.apply(0, new Array(localStorage.length)).map(function (o, i)" +
                "{ return localStorage.getItem(localStorage.key(i)); })"
            )
            driver.execute_script(script_js)
            printer.success("ok")
            return {'returnCode': Result.OK}
        except BaseException as err:
            printer.danger("FAILED")
            print(err)
            # sys.exit(1)
            return {'returnCode': Result.KO}

    def screen_shot(self, driver, file_name,is_server):
        """ screenshot """
        printer = InitPrinter()
        try:
            driver.get_screenshot_as_file(file_name)
            return Result.OK
        except BaseException as err:
            printer.danger("FAILED")
            print(err)
            if is_server is False:
                sys.exit(1)

    def click(self, driver, config, object_step):
        '''click '''
        printer = InitPrinter()
        by = SelBy()
        try:
            print(object_step["note"], end="->", flush=True)
            time.sleep(1)
            #for retrocompat
            if "xpath" in object_step:
                object_step["findBy"] = "XPATH"
                object_step["target"] = object_step["xpath"]
            driver.find_element(by.get_by(object_step["findBy"]),
                                object_step["target"]).click()
            printer.success("ok")
            return {'returnCode': Result.OK}
        except BaseException as err:
            printer.danger("FAILED")
            print(err)
            return {'returnCode': Result.KO}

    def select(self, driver, config, object_step):
        ''' select '''
        printer = InitPrinter()
        by = SelBy()
        print(object_step)
        try:
            print(object_step["note"], end="->", flush=True)
            time.sleep(1)
            #for retrocompat
            if "xpath" in object_step:
                object_step["findBy"] = "XPATH"
                object_step["target"] = object_step["xpath"]
            select = Select(
                driver.find_element(by.get_by(object_step["findBy"]),
                                    object_step["target"]))
            if "selectType" in object_step:
                if object_step["selectType"] == "label":
                    select.select_by_visible_text(object_step["value"])
                elif object_step["selectType"] == "value":
                    select.select_by_value(object_step["value"])
                elif object_step["selectType"] == "index":
                    select.select_by_index(object_step["value"])
                else:
                    printer.danger("selectType:" + object_step["selectType"] +
                                   " not supported in this moment")
            else:
                select.select_by_visible_text(object_step["value"])
            printer.success("ok")
            return {'returnCode': Result.OK}
        except BaseException as err:
            printer.danger("FAILED")
            printer.danger(err)
            return {'returnCode': Result.KO}

    def clear(self, driver, config, object_step):
        ''' clear '''
        printer = InitPrinter()
        by = SelBy()
        try:
            print(object_step["note"], end="->", flush=True)
            time.sleep(1)
            if "xpath" in object_step:
                object_step["findBy"] = "XPATH"
                object_step["target"] = object_step["xpath"]
            driver.find_element(by.get_by(object_step["findBy"]),
                                object_step["target"]).clear()
            printer.success("ok")
            return {'returnCode': Result.OK}
        except BaseException as err:
            printer.danger("FAILED")
            print(err)
            return {'returnCode': Result.KO}

    def send_keys(self, driver, config, object_step):
        ''' send keys '''
        printer = InitPrinter()
        selenium_key = EventKey()
        by = SelBy()
        try:
            string_to_input = object_step["text"]
            key = selenium_key.get_key(string_to_input)
            if key is None:
                if object_step["text"][:1] == "%":
                    string_to_input = config["json_config"][object_step["text"]
                                                            [1:]]
            else:
                string_to_input = key
            print(object_step["note"], end="->", flush=True)
            time.sleep(1)
            if "xpath" in object_step:
                object_step["findBy"] = "XPATH"
                object_step["target"] = object_step["xpath"]
            driver.find_element(
                by.get_by(object_step["findBy"]),
                object_step["target"]).send_keys(string_to_input)
            printer.success("ok")
            return {'returnCode': Result.OK}
        except BaseException as err:
            printer.danger("FAILED")
            print(err)
            # sys.exit(1)
            return {'returnCode': Result.KO}

    def wait_for_next_step(self, driver, config, object_step):
        ''' wait for next step'''
        by = SelBy()
        if "xpath" in object_step:
            object_step["findBy"] = "XPATH"
            object_step["target"] = object_step["xpath"]
        if (self.wait_for_next_step_real(
                driver,
                by.get_by(object_step["findBy"]),
                object_step["target"],
                object_step["note"],
        ) == Result.KO):
            return {'returnCode': Result.KO}
        return {'returnCode': Result.OK}

    def wait_for_next_step_real(self,
                                driver,
                                by,
                                target,
                                note,
                                wait_seconds=20):
        '''wait for next step'''
        failed = False
        printer = InitPrinter()
        try:
            print(note, end="->", flush=True)
            WebDriverWait(driver, wait_seconds).until(
                EC.presence_of_element_located((by, target)))
        except BaseException as err:
            printer.danger("FAILED")
            print(err)
            failed = True
            return Result.KO
        finally:
            if failed is False:
                printer.success("ok")
                return Result.OK
            return Result.KO

    def command(self, command, driver, obj_config, object_step):
        ''' command '''
        printer = InitPrinter()
        commands = {
            "wait_and_click": self.wait_and_click,
            "wait_for_next_step": self.wait_for_next_step,
            "wait_for_next_step_real": self.wait_for_next_step_real,
            "find_element_by_xpath": self.find_element_by_xpath,
            "find_elements_by_xpath": self.find_elements_by_xpath,
            "find_element": self.find_element_by_xpath,
            "find_elements": self.find_elements_by_xpath,
            "page_source": self.page_source,
            "switch_to_frame": self.switch_to_frame,
            "switch_to_default_content": self.switch_to_default_content,
            "find_object_element": self.find_object_element,
            "click_object": self.click_object,
            "click": self.click,
            "select": self.select,
            "clear": self.clear,
            "write": self.send_keys,
            "open_browser": self.open_browser,
            "write_localstorage": self.write_localstorage,
            "screen_shot": self.screen_shot,
            "sleep": self.sleep,
        }
        if command in commands.keys():
            return commands[command](driver, obj_config, object_step)
        printer.danger("Idelium Selenium | action non trovata:" + command)
        return None



"""System module."""
from __future__ import absolute_import
import sys
import importlib.util
from idelium._internal.commons.resultenum import Result
from idelium._internal.wrappers.ideliumselenium import IdeliumSelenium
from idelium._internal.wrappers.ideliumappium import IdeliumAppium

class StartManager:
    ''' Start manager '''
    @staticmethod
    def load_module(name):
        ''' load plugin '''
        print(name)
        name = name + ".plugin"
        print(name)
        mod = __import__(name, fromlist=["plugin"])
        return mod
    @staticmethod
    def get_wrapper(config):
        ''' return type of wrapper '''
        wrapper = None
        if config["isRealDevice"] is False:
            if config["is_debug"] is True:
                print("Using wrapper Selenium")
            wrapper = IdeliumSelenium()
        else:
            if config["is_debug"] is True:
                print("Using wrapper Appium")
            wrapper = IdeliumAppium()
        return wrapper

    @staticmethod
    def execute_step(driver, config):
        ''' Execute single step '''
        status = "1"
        step_failed = ""
        wrapper = config["wrapper"]
        printer = config["printer"]
        typeOfStep='seleniumOrAppium'
        postman_data=None        
        for object_step in config["json_step"]["steps"]:
            if status == "1":
                if object_step['stepType'] == 'postman_collection':
                    #postman=PostmanCollection()
                    #postman_data=postman.start_postman_test(object_step['collection'])
                    typeOfStep='postman'
                else:
                    return_object_step = wrapper.command(object_step["stepType"],
                                                    driver, config, object_step)
                    if return_object_step is not None:
                        if "config" in return_object_step.keys():
                            config = return_object_step["config"]
                        if "driver" in return_object_step.keys():
                            driver = return_object_step["driver"]
                        if return_object_step["returnCode"] == Result.KO:
                            status = "2"
                    else:
                        try:
                            module = importlib.import_module(
                                "plugin." + object_step["stepType"],
                                package=__package__)
                            params = None
                            if "params" in object_step:
                                params = object_step["params"]
                            plugin_response = module.init(driver,
                                                        config["json_config"],
                                                        params)
                            if plugin_response == Result.KO:
                                status = "2"
                                print(
                                    "Plugin response: " + object_step["note"],
                                    end="->",
                                    flush=True,
                                )
                                printer.danger("FAILED")
                            if plugin_response == Result.NA:
                                status = "5"
                                print(
                                    "Plugin response: " + object_step["note"],
                                    end="->",
                                    flush=True,
                                )
                                printer.warning("NA")
                        except BaseException as err:
                            printer.danger("----------")
                            print(err)
                            printer.danger("----------")
                            printer.danger(
                                "Warning stepType: " + object_step["stepType"] +
                                " not exist or there is an error in your extra module"
                            )
                            if config['ideliumServer'] is False:
                                sys.exit(1)
                            else:
                                status=2
                if status == "2":
                    step_failed = object_step
            else:
                printer.danger(object_step["stepType"] + ": skipped")
        return {"driver": driver, "status": status, "step_failed": step_failed, 'type': typeOfStep, 'postman_data': postman_data}

    def execute_single_step(self, test_configurations, config):
        """ execute single page """
        printer = config["printer"]
        driver = None

        if config["isRealDevice"] is False:
            if config["is_debug"] is True:
                print("Using wrapper Selenium")
            wrapper = IdeliumSelenium()
        else:
            if config["is_debug"] is True:
                print("Using wrapper Appium")
            wrapper = IdeliumAppium()
        for file_step_name in config["file_steps"].split(","):
            try:
                json_step = test_configurations["steps"][file_step_name]
                printer.underline(json_step["name"])
                config["wrapper"] = wrapper
                config["printer"] = printer
                config["json_step"] = json_step
                object_return = self.execute_step(driver,
                                                 config)
                driver = object_return["driver"]
                string_to_show = (file_step_name + " the return value " +
                                object_return["status"])
                if object_return["status"] == "1":
                    printer.success(string_to_show)
                elif object_return["status"] == "5":
                    printer.warning(string_to_show)
                else:
                    printer.danger(string_to_show)

            except BaseException as err:
                printer.danger("---------- Execute step ------")
                print(err)
                printer.danger("----------")
                printer.danger("Warning, the file step: " + file_step_name +
                               " not exist or is not a json (err 2)")
                if config['ideliumServer'] is False:
                    sys.exit(1)



import json
import ntpath
import magic
import os
from io import BytesIO

def extract_dict_from_raw_mode_data(raw):
    """extract json to dictionay

    :param raw: jsondata
    :return: :extracted dict
    """
    try:
        return json.loads(raw)
    except json.decoder.JSONDecodeError:
        return {}


def exctact_dict_from_files(data):
    """extract files from dict data.

    :param data: [{"key":"filename", "src":"relative/absolute path to file"}]
    :return: :tuple of file metadata for requests library
    """
    if not os.path.isfile(data['src']):
        raise Exception(
            'File '+data['src']+' does not exists')
    mime = magic.Magic(mime=True)
    file_mime = mime.from_file(data['src'])
    file_name = ntpath.basename(data['src'])
    with open(data['src'], 'rb') as fs:
        bs = BytesIO(fs.read()) # read bytes from file into memory
    return (file_name, bs, file_mime, {
        'Content-Disposition': 'form-data; name="'+data['key']+'"; filename="' + file_name + '"',
        'Content-Type': file_mime})


def extract_dict_from_formdata_mode_data(formdata):
    data = {}
    files = {}
    try:
        for row in formdata:
            if row['type'] == "text":
                data[row['key']] = row['value']
            if row['type'] == "file":
                files[row['key']] = exctact_dict_from_files(row)
        return data, files
    except Exception:
        print("extact from formdata_mode_data error occurred: ")
        return data, files


def extract_dict_from_raw_headers(raw):
    d = {}
    for header in raw.split('\n'):
        try:
            key, value = header.split(': ')
            d[key] = value
        except ValueError:
            continue

    return d


def extract_dict_from_headers(data):
    d = {}
    for header in data:
        try:
            if 'disabled' in header and header['disabled'] == True:
                continue
            d[header['key']] = header['value']
        except ValueError:
            continue

    return d


def format_object(o, key_values, is_graphql=False):
    print(o)
    if isinstance(o, str):
        try:
            if is_graphql:
                return o

            return o.replace('{{', '{').replace('}}', '}').format(**key_values)

        except KeyError as e:
            raise KeyError(
                "Except value %s in PostPython environment variables.\n Environment variables are %s" % (e, key_values))
    elif isinstance(o, dict):
        return format_dict(o, key_values, is_graphql)
    elif isinstance(o, list):
        return [format_object(oo, key_values, is_graphql) for oo in o]
    elif isinstance(o, object):
        return o


def format_dict(d, key_values, is_graphql):
    kwargs = {}
    for k, v in d.items():
        kwargs[k] = format_object(v, key_values, is_graphql)
    return kwargs



"""System module."""
from __future__ import absolute_import
import sys
import time
import base64
from appium import webdriver
from idelium._internal.commons.ideliumprinter import InitPrinter
from idelium._internal.commons.androideventkey import EventKey
from idelium._internal.commons.resultenum import Result

class IdeliumAppium():
    ''' Idelium Appium'''
    @staticmethod
    def wait_for_elements(driver,config, object_step):
        ''' wait for elements'''
        wait_time=5
        if 'waitTime' in config['json_config']:
            wait_time=config['json_config']['waitTime']
        print ("Waiting for Login dialog to open, max wait =" + str(wait_time) + " seconds")
        timeout = time.time() + wait_time   # Timer based on wait_time to prevent infinite loops
        element=None
        while True:
            time.sleep(1)   # Prevent CPU slamming with short timeout between loops
            if time.time() > timeout:
                print ('timeout exception')
                break
            try:
                if 'xpath' in object_step:
                    element = driver.find_element_by_xpath(
                        object_step['xpath'])
                elif 'native_interface_element' in object_step:
                    native_interface_element=config['appiumDesiredCaps']['appPackage'] + ":id/"
                    + object_step['native_interface_element']
                    element = driver.find_element_by_id(native_interface_element)
                break
            except BaseException as err:
                print ('still waiting')
                print (err)
        return element
    @staticmethod
    def connect_appium(driver,config,object_step):
        '''Connect appium '''
        return_code=Result.OK
        driver=None
        printer=InitPrinter()
        if config['is_debug'] is True:
            print ('try to connect:' + config['appiumServer'])
        print (object_step['note'],end="->", flush=True)
        try:
            driver = webdriver.Remote(config['appiumServer'],
                                      config['appiumDesiredCaps'],
                                      keep_alive=False)
            printer.success('ok')
            contexts=driver.contexts
            printer.warning ('Context name:')
            index=0
            for i in contexts:
                index =index + 1
                printer.warning (str(index) + ")" + i)
        except BaseException as err:
            printer.danger('ko')
            print (err)
            config['json_step']['attachScreenshot'] = False
            config['json_step']['failedExit'] = False
            printer.danger('Verify if Appium server is running')
            printer.danger('The test is stopped')
            if config['ideliumServer'] is False:
                sys.exit(1)
            return_code=Result.KO
        return {"driver" : driver,"config" : config, "returnCode" : return_code}
    def appium_send_keys(self,driver,config,object_step):
        ''' appium_send_keys '''
        return_code=Result.OK
        printer=InitPrinter()
        print (object_step['note'],end="->", flush=True)
        element=self.wait_for_elements(driver,config,object_step)
        if element is None:
            return_code=Result.KO
        else:
            if config['json_config']['appiumDesiredCaps']['platformName'] == 'android':
                element.click()
                time.sleep(1)
                event_key=EventKey()
                for string in object_step['keys']:
                    for key_command in event_key.get_array_of_char(string):
                        array_command=key_command.split(',')
                        if len(array_command) == 1:
                            driver.press_keycode(key_command)
                        else:
                            driver.press_keycode(array_command[0],array_command[1])
            else:
                for string in object_step['keys']:
                    element.send_keys(string)
        return {"returnCode" : return_code}
    def appium_click(self,driver,config,object_step):
        ''' appium click'''
        return_code=Result.OK
        printer=InitPrinter()
        print (object_step['note'],end="->", flush=True)
        element=self.wait_for_elements(driver,config,object_step)
        if element is None:
            return_code=Result.KO
        else:
            element.click()

        return {"returnCode" : return_code}




    def appium_switch_context(self,driver,config,object_step):
        ''' appium_switch_context '''
        return_code=Result.OK
        printer=InitPrinter()
        print (object_step['note'],end="->", flush=True)
        try:
            driver.switch_to.context(object_step['contextName'])
            driver.wait_activity
            printer.success('ok')
        except BaseException as err:
            return_code=Result.KO
            printer.danger('ko')
            print(err)
        return {"returnCode": return_code}

    def appium_execute_script(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/mobile-command/
        """
        return_code=Result.OK
        driver.execute_script(object_step['script'])
        return return_code
    def appium_desired_capabilities(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/session/get/
        """
        return driver.desired_capabilities()
    def appium_back(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/session/back/
        """
        return_code=Result.OK
        driver.back()
        return return_code
    def appium_page_source(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/session/source/
        """
        print (driver.page_source)
        return driver.page_source
    def appium_set_page_load_timeout(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/session/timeouts/timeouts/
        """
        return_code=Result.OK
        driver.set_page_load_timeout(object_step['milliseconds'])
        return return_code
    def appium_implicitly_wait(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/session/timeouts/implicit-wait/
        """
        return_code=Result.OK
        driver.implicitly_wait(object_step['milliseconds'])
        return return_code
    def appium_set_script_timeout(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/session/timeouts/async-script/
        """
        return_code=Result.OK
        driver.set_script_timeout(object_step['milliseconds'])
        return return_code
    def appium_orientation(self,driver,config,object_step):
        """
            orientation: LANDSCAPE,PORTRAIT
            for more info:
            https://appium.io/docs/en/commands/session/orientation/set-orientation/
        """
        return_code=Result.OK
        driver.orientation(object_step['orientation'])
        return return_code
    def appium_location(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/session/geolocation/get-geolocation/
        """
        return driver.location()
    def appium_orientation(self,driver,config,object_step):
        """
            orientation: LANDSCAPE,PORTRAIT
            for more info:
            https://appium.io/docs/en/commands/session/geolocation/set-geolocation/
        """
        return_code=Result.OK
        driver.set_location(object_step['latitude'],object_step['ongitude'],object_step['altitude'])
        return return_code
    def appium_log_types(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/session/logs/get-log-types/
        """
        return driver.log_types()
    def appium_get_log(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/session/logs/get-log/
        """
        return driver.get_log(object_step['typeString'])
    def appium_update_settings(self,driver,config,object_step):
        """
            orientation: LANDSCAPE,PORTRAIT
            for more info:
            https://appium.io/docs/en/commands/session/settings/update-settings/
        """
        return_code=Result.OK
        driver.update_settings(object_step['jsonSettings'])
        return return_code
    def appium_get_settings(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/session/settings/get-settings/
        """
        return driver.get_settings
    def start_start_activity(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/activity/start-activity/
        """
        return driver.start_activity(object_step['jsonActivityParameters'])
    def appium_current_activity(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/activity/current-activity/
        """
        return driver.current_activity
    def appium_current_package(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/app/install-app/
        """
        printer=InitPrinter()
        try:
            driver.install_app(object_step['appPath'])
            return Result.OK
        except BaseException as err:
            printer.danger('FAILED')
            print(err)
            if config['ideliumServer'] is False:
                sys.exit(1)
            return Result.KO
    def appium_is_app_installed(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/app/is-app-installed/
        """
        return driver.is_app_installed(object_step['appPackage'])
    def appium_launch_app(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/app/launch-app/
        """
        return_code=Result.OK
        driver.launch_app()
        return return_code
    def appium_background_app(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/app/background-app/
        """
        return_code=Result.OK
        driver.background_app(object_step['seconds'])
        return return_code
    def appium_close_app(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/app/close-app/
        """
        return_code=Result.OK
        driver.close_app()
        return return_code
    def appium_reset_app(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/app/reset-app/
        """
        return_code=Result.OK
        driver.reset()
        return return_code
    def appium_remove_app(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/app/remove-app/
        """
        return_code=Result.OK
        driver.remove_app()
        return return_code
    def appium_activate_app(self,driver,config,object_step):
        """
            examples:
            driver.appium_activate_app('com.apple.Preferences')
            driver.appium_activate_app('io.appium.android.apis')
            for more info:
            https://appium.io/docs/en/commands/device/app/activate-app/
        """
        return_code=Result.OK
        driver.activate_app(object_step['bundleId'])
        return return_code
    def appium_terminate_app(self,driver,config,object_step):
        """
            examples:
            driver.appium_terminate_app('com.apple.Preferences')
            driver.appium_terminate_app('io.appium.android.apis')
            for more info:
            https://appium.io/docs/en/commands/device/app/terminate-app/
        """
        return_code=Result.OK
        driver.terminate_app(object_step['bundleId'])
        return return_code
    def appium_query_app_state(self,driver,config,object_step):
        """
            examples:
            driver.appium_query_app_state('com.apple.Preferences')
            driver.appium_query_app_state('io.appium.android.apis')
            for more info:
            https://appium.io/docs/en/commands/device/app/app-state/
        """
        return_code=Result.OK
        driver.query_app_state(object_step['bundleId'])
        return return_code
    def appium_app_strings(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/app/get-app-strings/
        """
        return_code=Result.OK
        driver.app_strings(object_step['language'],object_step['pathFile'])
        return return_code
    def appium_end_test_coverage(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/app/end-test-coverage/
        """
        return_code=Result.OK
        driver.end_test_coverage(object_step['intent'],object_step['path'])
        return return_code
    def appium_set_clipboard(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/clipboard/set-clipboard/
        """
        return_code=Result.OK
        driver.set_clipboard(object_step['string'])
        return return_code
    def appium_set_power_ac(self,driver,config,object_step):
        """
            Examples:
            self.driver.set_power_ac(Power.AC_OFF)
            for more info:
            https://appium.io/docs/en/commands/device/emulator/power_ac/
        """
        return_code=Result.OK
        driver.set_power_ac(object_step['powerOnOff'])
        return return_code
    def appium_set_power_capacity(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/emulator/power_capacity/
        """
        return_code=Result.OK
        driver.set_power_capacity(object_step['percent'])
        return return_code
    def appium_push_file(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/files/push-file/
        """
        return_code=Result.OK
        driver.push_file(object_step['path'],object_step['data'])
        return return_code
    def appium_pull_file(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/files/pull-file/
        """
        return driver.pull_file(object_step['path'])
    def appium_pull_folder(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/files/pull-folder/
        """
        return driver.pull_folder(object_step['path'])
    def appium_shake(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/interactions/shake/
        """
        return_code=Result.OK
        driver.shake()
        return return_code
    def appium_lock(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/interactions/lock/
        """
        return_code=Result.OK
        driver.lock()
        return return_code
    def appium_unlock(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/interactions/unlock/
        """
        return_code=Result.OK
        driver.unlock()
        return return_code
    def appium_is_locked(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/interactions/is-locked/
        """
        return_code=Result.OK
        driver.is_locked()
        return return_code
    def appium_press_keycode(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/keys/press-keycode/
        """
        return_code=Result.OK
        driver.press_keycode(object_step['keyCode'])
        return return_code
    def appium_long_press_keycode(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/keys/long_press-keycode/
        """
        return_code=Result.OK
        driver.long_press_keycode(object_step['keyCode'])
        return return_code
    def appium_hide_keyboard(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/keys/hide-keyboard/
        """
        return_code=Result.OK
        driver.hide_keyboard()
        return return_code
    def appium_is_keyboard_shown(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/keys/is-keyboard-shown/
        """
        return driver.is_keyboard_shown()
    def appium_toggle_wifi(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/network/toggle-wifi/
        """
        return_code=Result.OK
        driver.toggle_wifi()
        return return_code
    def appium_toggle_location_services(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/network/toggle-location-services/
        """
        return_code=Result.OK
        driver.toggle_location_services()
        return return_code
    def appium_send_sms(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/network/send-sms/
        """
        return_code=Result.OK
        driver.send_sms(object_step['phoneNumber'],object_step['message'])
        return return_code
    def appium_make_gsm_call(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/network/gsm-call/
        """
        return_code=Result.OK
        driver.make_gsm_call(object_step['phoneNumber'],object_step['gsmAction'])
        return return_code
    def appium_set_gsm_signal(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/network/gsm-signal/
        """
        return_code=Result.OK
        driver.set_gsm_signal(object_step['gsmSignalStrength'])
        return return_code
    def appium_set_gsm_voice(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/network/gsm-voice/
        """
        return_code=Result.OK
        driver.set_gsm_voice(object_step['gsmVoiceState'])
        return return_code
    def appium_set_network_speed(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/network/network-speed/
        """
        return_code=Result.OK
        driver.set_network_speed(object_step['netSpeed'])
        return return_code
    def appium_get_performance_data(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/performance-data/get-performance-data/
        """
        return_code=Result.OK
        driver.get_performance_data(object_step['packageName'],
                                    object_step['dataType'],
                                    object_step['dataReatTimeOut'])
        return return_code
    def appium_get_performance_data_types(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/performance-data/performance-data-types/
        """
        return driver.get_performance_data_types()
    def appium_start_recording_screen(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/recording-screen/start-recording-screen/
        """
        return_code=Result.OK
        if object_step['options'] is None:
            driver.start_recording_screen()
        else:
            driver.start_recording_screen(object_step['options'])
        return return_code
    def appium_stop_recording_screen(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/recording-screen/stop-recording-screen/
        """
        return_code=Result.OK
        driver.stop_recording_screen()
        return return_code
    def appium_touch_id(self,driver,config,object_step):
        """
            self.driver.touch_id(false); # Simulates a failed touch
            self.driver.touch_id(true); # Simulates a passing touch
            for more info:
            https://appium.io/docs/en/commands/device/simulator/touch-id/
        """
        return_code=Result.OK
        driver.touch_id(object_step['touch'])
        return return_code
    def appium_toggle_touch_id_enrollment(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/simulator/toggle-touch-id-enrollment/
        """
        return_code=Result.OK
        driver.toggle_touch_id_enrollment()
        return return_code
    def appium_open_notifications(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/system/open-notifications/
        """
        return_code=Result.OK
        driver.open_notifications()
        return return_code
    def appium_get_system_bars(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/system/system-bars/
        """
        return_code=Result.OK
        driver.get_system_bars()
        return return_code
    def appium_get_system_time(self,driver,config ,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/system/system-time/
        """
        return_string=None
        if object_step['date'] is None:
            return_string=driver.get_device_time()
        else:
            return_string=driver.get_device_time(object_step['date'])
        return return_string
    def appium_get_device_density(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/system/display-density/
        """
        return_code=Result.OK
        driver.get_device_density()
        return return_code
    def appium_finger_print(self,driver,config,object_step):
        """
            for more info:
            https://appium.io/docs/en/commands/device/authentication/finger-print/
        """
        return_code=Result.OK
        driver.finger_print(object_step['number'])
        return return_code
    def appium_find_element_by_accessibility_id(self,driver,config,object_step):
        """
            for more info:
           https://appium.io/docs/en/commands/element/find-element/
        """
        return driver.find_element_by_accessibility_id(object_step['accessibilityId'])
    def appium_switch_to(self,driver,config,object_step):
        """
        """
        return driver.switch_to()
    def screen_shot(self,driver,file_name):
        """
                screenshot appium
        """
        printer=InitPrinter()
        try:
            current_context=driver.current_context
            fake_object_step={}
            fake_object_step['contextName']='NATIVE_APP'
            fake_object_step['note']='take screenshot'
            self.appium_switch_context(driver,None,fake_object_step)
            screenshot=driver.get_screenshot_as_base64()
            fake_object_step['contextName']=current_context
            self.appium_switch_context(driver,None,fake_object_step)
            screenshot_data = base64.b64decode(screenshot)
            new_file = open(file_name, "wb")
            new_file.write(screenshot_data)
            new_file.close()
            return Result.OK
        except BaseException as err:
            printer.danger('FAILED APP SCREENSHOT')
            print(err)
            return Result.KO

    def  command (self,command,driver,obj_config,object_step):
        '''Command'''
        printer=InitPrinter()
        commands= {
                "connect_appium" : self.connect_appium,
                "appium_send_keys" : self.appium_send_keys,
                "appium_send_keys_xpath": self.appium_send_keys,
                "appium_click" : self.appium_click,
                "appium_click_xpath" : self.appium_click,
                "appium_switch_context" : self.appium_switch_context,
                "appium_execute_script" : self.appium_execute_script,
                "appium_desired_capabilities" : self.appium_desired_capabilities,
                "appium_back" : self.appium_back,
                "appium_page_source" : self.appium_page_source,
                "appium_set_page_load_timeout" : self.appium_set_page_load_timeout,
                "appium_implicitly_wait" : self.appium_implicitly_wait,
                "appium_set_script_timeout" : self.appium_set_script_timeout,
                "appium_location" : self.appium_location,
                "appium_orientation" : self.appium_orientation,
                "appium_log_types" : self.appium_log_types,
                "appium_get_log" : self.appium_get_log,
                "appium_update_settings" : self.appium_update_settings,
                "appium_get_settings" : self.appium_get_settings,
                "appium_start_start_activity" : self.start_start_activity,
                "appium_current_activity" : self.appium_current_activity,
                "appium_current_package" : self.appium_current_package,
                "appium_is_app_installed" : self.appium_is_app_installed,
                "appium_launch_app" : self.appium_launch_app,
                "appium_background_app" : self.appium_background_app,
                "appium_close_app" : self.appium_close_app,
                "appium_reset_app" : self.appium_reset_app,
                "appium_remove_app" : self.appium_remove_app,
                "appium_activate_app" : self.appium_activate_app,
                "appium_terminate_app" : self.appium_terminate_app,
                "appium_query_app_state" : self.appium_query_app_state,
                "appium_app_strings" : self.appium_app_strings,
                "appium_end_test_coverage" : self.appium_end_test_coverage,
                "appium_set_clipboard" : self.appium_set_clipboard,
                "appium_set_power_ac" : self.appium_set_power_ac,
                "appium_set_power_capacity" : self.appium_set_power_capacity,
                "appium_push_file" : self.appium_push_file,
                "appium_pull_file" : self.appium_pull_file,
                "appium_pull_folder" : self.appium_pull_folder,
                "appium_shake" : self.appium_shake,
                "appium_lock" : self.appium_lock,
                "appium_unlock" : self.appium_unlock,
                "appium_is_locked" : self.appium_is_locked,
                "appium_press_keycode" : self.appium_press_keycode,
                "appium_long_press_keycode" : self.appium_long_press_keycode,
                "appium_hide_keyboard" : self.appium_hide_keyboard,
                "appium_is_keyboard_shown" : self.appium_is_keyboard_shown,
                "appium_toggle_wifi" : self.appium_toggle_wifi,
                "appium_toggle_location_services" : self.appium_toggle_location_services,
                "appium_send_sms" : self.appium_send_sms,
                "appium_make_gsm_call" : self.appium_make_gsm_call,
                "appium_set_gsm_signal" : self.appium_set_gsm_signal,
                "appium_set_gsm_voice" : self.appium_set_gsm_voice,
                "appium_set_network_speed" : self.appium_set_network_speed,
                "appium_get_performance_data" : self.appium_get_performance_data,
                "appium_get_performance_data_types" : self.appium_get_performance_data_types,
                "appium_start_recording_screen" : self.appium_start_recording_screen,
                "appium_stop_recording_screen" : self.appium_stop_recording_screen,
                "appium_touch_id" : self.appium_touch_id,
                "appium_toggle_touch_id_enrollment" : self.appium_toggle_touch_id_enrollment,
                "appium_open_notifications" : self.appium_open_notifications,
                "appium_get_system_bars" : self.appium_get_system_bars,
                "appium_get_system_time" : self.appium_get_system_time,
                "appium_get_device_density" : self.appium_get_device_density,
                "appium_finger_print" : self.appium_finger_print,
                "appium_find_element_by_accessibility_id" :
                    self.appium_find_element_by_accessibility_id,
                "appium_switch_to" : self.appium_switch_to,
        }
        if command in commands.keys():
            return commands[command](driver,obj_config,object_step)
        printer.danger ('Idelium Appium | action non trovata:' + command)
        return None
